import argparse
import os
from datetime import datetime
from pathlib import Path
from uuid import uuid4

parser = argparse.ArgumentParser("train")
parser.add_argument("--training_data", type=str, help="Path to training data")
parser.add_argument("--max_epocs", type=int, help="Max # of epocs for the training")
parser.add_argument("--learning_rate", type=float, help="Learning rate")
parser.add_argument("--learning_rate_schedule", type=str, help="Learning rate schedule")
parser.add_argument("--model_output", type=str, help="Path of output model")
parser.add_argument("--reports", type=str, help="Path of reports")

args = parser.parse_args()

print("hello training world...")

lines = [
    f"Training data path: {args.training_data}",
    f"Max epocs: {args.max_epocs}",
    f"Learning rate: {args.learning_rate}",
    f"Learning rate: {args.learning_rate_schedule}",
    f"Model output path: {args.model_output}",
    f"Model output path: {args.reports}",
]

for line in lines:
    print(line)

print("mounted_path files: ")
arr = os.listdir(args.training_data)
print(arr)

for filename in arr:
    print("reading file: %s ..." % filename)
    with open(os.path.join(args.training_data, filename), "r") as handle:
        print(handle.read())


# Do the train and save the trained model as a file into the output folder.
# Here only output a dummy data for demo.
curtime = datetime.now().strftime("%b-%d-%Y %H:%M:%S")
model = f"This is a dummy model with id: {str(uuid4())} generated at: {curtime}\n"
(Path(args.model_output) / "model.txt").write_text(model)

curtime = datetime.now().strftime("%b-%d-%Y %H:%M:%S")
model = f"This is a dummy report with id: {str(uuid4())} generated at: {curtime}\n"
(Path(args.reports) / "report.txt").write_text(model)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from typing import Any, Dict, Optional

from azure.ai.ml._schema._deployment.online.event_hub_schema import EventHubSchema
from azure.ai.ml.constants._common import BASE_PATH_CONTEXT_KEY
from azure.ai.ml.entities._deployment.oversize_data_config import OversizeDataConfig


class EventHub:
    """Event Hub deployment entity

    :param namespace: Name space of eventhub, provided in format of "{namespace}.{name}".
    :type namespace: str
    :param oversize_data_config: Oversized payload body configurations.
    :type oversize_data_config: OversizeDataConfig

    """

    # pylint: disable=unused-argument
    def __init__(
        self, namespace: Optional[str] = None, oversize_data_config: Optional[OversizeDataConfig] = None, **kwargs: Any
    ):
        self.namespace = namespace
        self.oversize_data_config = oversize_data_config

    def _to_dict(self) -> Dict:
        # pylint: disable=no-member
        res: dict = EventHubSchema(context={BASE_PATH_CONTEXT_KEY: "./"}).dump(self)
        return res



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=arguments-renamed

import logging
from typing import Optional

from azure.ai.ml._restclient.v2022_05_01.models import ContainerResourceSettings
from azure.ai.ml.entities._mixins import RestTranslatableMixin

module_logger = logging.getLogger(__name__)


class ResourceSettings(RestTranslatableMixin):
    """Resource settings for a container.

    This class uses Kubernetes Resource unit formats. For more information, see
    https://kubernetes.io/docs/concepts/configuration/manage-resources-containers/.

    :param cpu: The CPU resource settings for a container.
    :type cpu: Optional[str]
    :param memory: The memory resource settings for a container.
    :type memory: Optional[str]
    :param gpu: The GPU resource settings for a container.
    :type gpu: Optional[str]

    .. admonition:: Example:

        .. literalinclude:: ../samples/ml_samples_misc.py
            :start-after: [START resource_requirements_configuration]
            :end-before: [END resource_requirements_configuration]
            :language: python
            :dedent: 8
            :caption: Configuring ResourceSettings for a Kubernetes deployment.
    """

    def __init__(self, cpu: Optional[str] = None, memory: Optional[str] = None, gpu: Optional[str] = None) -> None:
        self.cpu = cpu
        self.memory = memory
        self.gpu = gpu

    def _to_rest_object(self) -> ContainerResourceSettings:
        return ContainerResourceSettings(cpu=self.cpu, memory=self.memory, gpu=self.gpu)

    @classmethod
    def _from_rest_object(cls, settings: ContainerResourceSettings) -> Optional["ResourceSettings"]:
        return (
            ResourceSettings(
                cpu=settings.cpu,
                memory=settings.memory,
                gpu=settings.gpu,
            )
            if settings
            else None
        )

    def _merge_with(self, other: Optional["ResourceSettings"]) -> None:
        if other:
            self.cpu = other.cpu or self.cpu
            self.memory = other.memory or self.memory
            self.gpu = other.gpu or self.gpu

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, ResourceSettings):
            return NotImplemented
        if not other:
            return False
        # only compare mutable fields
        return self.cpu == other.cpu and self.memory == other.memory and self.gpu == other.gpu

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)



from hmac import new
from pathlib import Path
from marshmallow.exceptions import ValidationError
import pytest
from test_utilities.utils import verify_entity_load_and_dump

from azure.ai.ml import load_connection
from azure.ai.ml._restclient.v2024_04_01_preview.models import ConnectionAuthType, ConnectionCategory
from azure.ai.ml._utils.utils import camel_to_snake
from azure.ai.ml.entities import WorkspaceConnection, AzureOpenAIConnection
from azure.ai.ml.constants._common import ConnectionTypes, CognitiveServiceKinds

from azure.ai.ml.entities._credentials import (
    AccessKeyConfiguration,
    ApiKeyConfiguration,
    ManagedIdentityConfiguration,
    NoneCredentialConfiguration,
    PatTokenConfiguration,
    SasTokenConfiguration,
    ServicePrincipalConfiguration,
    UsernamePasswordConfiguration,
    _BaseIdentityConfiguration,
)

from azure.ai.ml.entities import (
    AzureBlobStoreConnection,
    MicrosoftOneLakeConnection,
    AzureOpenAIConnection,
    AzureAIServicesConnection,
    AzureAISearchConnection,
    AzureContentSafetyConnection,
    AzureSpeechServicesConnection,
    APIKeyConnection,
    OpenAIConnection,
    SerpConnection,
    ServerlessConnection,
)


@pytest.mark.unittest
@pytest.mark.core_sdk_test
class TestWorkspaceConnectionEntity:
    def assert_connections_are_similar(self, conn1: WorkspaceConnection, conn2: WorkspaceConnection):
        """Helper functiom which runs a bunch of assertions to check that the two
        inputted connections are roughly equivalent. Does not check certain features
        that are specific to some connection subclasses.

        :param conn1: The first connection to compare.
        :type conn1: ~azure.ai.ml.entities.WorkspaceConnection
        :param conn2: The second connection to compare.
        :type conn2: ~azure.ai.ml.entities.WorkspaceConnection
        """

        # Check them individually to simplify debugging.
        assert conn1 is not None
        assert conn2 is not None
        assert conn1.name == conn2.name
        assert type(conn1) == type(conn2)
        assert conn1.type == conn2.type
        assert conn1.target == conn2.target
        assert conn1.tags == conn2.tags  # Check deprecated access
        assert conn1.metadata == conn2.metadata  # This helpfully compares hidden metadata for subclasses.
        assert conn1.credentials == conn2.credentials

    def check_rest_conversion_stable(self, conn: WorkspaceConnection):
        """Helper function which converts an inputted connection to its rest
        equivalent and back again, then checks that they're still the same.

        :param conn: The connection to check
        :type conn: ~azure.ai.ml.entities.WorkspaceConnection
        """

        rest_conn = conn._to_rest_object()
        new_conn = WorkspaceConnection._from_rest_object(rest_obj=rest_conn)
        # equalize names since rest conversions don't keep it.
        new_conn.name = conn.name
        self.assert_connections_are_similar(conn, new_conn)

    def check_dict_conversion_stable(self, conn: WorkspaceConnection):
        """Helper function which converts and inputted connection to its dictionary
        equivalent and back again, then checks that they're still the same.

        :param conn: The connection to check
        :type conn: ~azure.ai.ml.entities.WorkspaceConnection
        """

        conn_dictionary = conn._to_dict()
        new_conn = WorkspaceConnection._load(conn_dictionary)
        self.assert_connections_are_similar(conn, new_conn)

    def check_all_conversions_stable(self, conn: WorkspaceConnection):
        """Helper function which checks that a connection is stable when converting
        to and from rest and dictionary representations.

        :param conn: The connection to check
        :type conn: ~azure.ai.ml.entities.WorkspaceConnection
        """

        self.check_rest_conversion_stable(conn)
        self.check_dict_conversion_stable(conn)

    def test_workspace_connection_constructor(self):
        ws_connection = WorkspaceConnection(
            target="dummy_target",
            type=camel_to_snake(ConnectionCategory.PYTHON_FEED),
            credentials=PatTokenConfiguration(pat="dummy_pat"),
            name="dummy_connection",
            metadata=None,
        )

        assert ws_connection.name == "dummy_connection"
        assert ws_connection.type == camel_to_snake(ConnectionCategory.PYTHON_FEED)
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.PAT)
        assert ws_connection.credentials.pat == "dummy_pat"
        assert ws_connection.target == "dummy_target"
        assert ws_connection.metadata == {}

    def test_workspace_tags_and_metadata_precedence(self):

        ws_connection = WorkspaceConnection(
            target="dummy_target",
            type=camel_to_snake(ConnectionCategory.PYTHON_FEED),
            credentials=PatTokenConfiguration(pat="dummy_pat"),
            name="dummy_connection",
            metadata={"value": "metadata", "a": "b"},
            tags={"value": "tags", "c": "d"},
        )
        assert ws_connection.metadata == {"value": "metadata", "a": "b", "c": "d"}

    # New connection tests - these connections have had their lists of acceptable credentials validated in 2024

    def test_workspace_connection_entity_load_and_dump(self):
        def simple_workspace_connection_validation(ws_connection):
            assert ws_connection.name == "test_ws_conn_git_pat"
            assert ws_connection.target == "https://test-git-feed.com"
            assert ws_connection.type == camel_to_snake(ConnectionCategory.GIT)
            assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.PAT)
            assert ws_connection.credentials.pat == "dummy_pat"
            assert ws_connection.metadata == {}

        verify_entity_load_and_dump(
            load_connection,
            simple_workspace_connection_validation,
            "./tests/test_configs/connection/git_pat.yaml",
        )

    def test_blob_storage(self):
        ws_connection = load_connection(source="./tests/test_configs/connection/blob_store_acc_key.yaml")
        assert type(ws_connection) == AzureBlobStoreConnection
        assert ws_connection.type == camel_to_snake(ConnectionCategory.AZURE_BLOB)
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.ACCOUNT_KEY)
        assert ws_connection.credentials.account_key == "9876"
        assert ws_connection.name == "test_ws_conn_blob_store1"
        assert ws_connection.url == "my_endpoint"
        assert ws_connection.metadata["four"] == "five"
        assert ws_connection.container_name == "some_container"
        assert ws_connection.account_name == "some_account"
        self.check_all_conversions_stable(ws_connection)

        ws_connection = load_connection(source="./tests/test_configs/connection/blob_store_sas.yaml")
        assert type(ws_connection) == AzureBlobStoreConnection
        assert ws_connection.type == camel_to_snake(ConnectionCategory.AZURE_BLOB)
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.SAS)
        assert ws_connection.credentials.sas_token == "some_pat"
        assert ws_connection.name == "test_ws_conn_blob_store2"
        self.check_all_conversions_stable(ws_connection)

        ws_connection = load_connection(source="./tests/test_configs/connection/blob_store_entra.yaml")
        assert type(ws_connection) == AzureBlobStoreConnection
        assert ws_connection.type == camel_to_snake(ConnectionCategory.AZURE_BLOB)
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.AAD)
        assert ws_connection.name == "test_ws_conn_blob_store3"
        self.check_all_conversions_stable(ws_connection)

    def test_alds_gen2(self):
        ws_connection = load_connection(source="./tests/test_configs/connection/alds_gen2_sp.yaml")
        assert type(ws_connection) == WorkspaceConnection
        assert ws_connection.type == ConnectionTypes.AZURE_DATA_LAKE_GEN_2
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.SERVICE_PRINCIPAL)
        assert ws_connection.credentials.tenant_id == "1234"
        assert ws_connection.credentials.client_id == "12345"
        assert ws_connection.credentials.client_secret == "123456"
        assert ws_connection.credentials.authority_url == "https://login.microsoftonline.com"
        assert ws_connection.name == "test_gen2_conn1"
        assert ws_connection.target == "my_endpoint"
        assert ws_connection.metadata["four"] == "five"
        self.check_dict_conversion_stable(ws_connection)
        # Auth url doesn't get copied back by rest conversion intentionally, so black it here
        # to simplify comparison.
        ws_connection.credentials.authority_url = ""
        self.check_rest_conversion_stable(ws_connection)

        ws_connection = load_connection(source="./tests/test_configs/connection/alds_gen2_entra.yaml")
        assert type(ws_connection) == WorkspaceConnection
        assert ws_connection.type == ConnectionTypes.AZURE_DATA_LAKE_GEN_2
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.AAD)
        assert ws_connection.name == "test_gen2_conn2"
        self.check_all_conversions_stable(ws_connection)

    def test_one_lake(self):
        # Note: also tests SP credential.
        ws_connection = load_connection(source="./tests/test_configs/connection/one_lake_with_name.yaml")
        assert type(ws_connection) == MicrosoftOneLakeConnection
        assert ws_connection.type == camel_to_snake(ConnectionCategory.AZURE_ONE_LAKE)
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.SERVICE_PRINCIPAL)
        assert ws_connection.credentials.tenant_id == "1234"
        assert ws_connection.credentials.client_id == "12345"
        assert ws_connection.credentials.client_secret == "123456"
        assert ws_connection.credentials.authority_url == "https://login.microsoftonline.com"
        assert ws_connection.name == "one_lake_with_name"
        assert ws_connection.target == "https://www.endpoint.com/the_workspace_name/my_lake_name.Lakehouse"
        self.check_dict_conversion_stable(ws_connection)
        # Auth url doesn't get copied back by rest conversion intentionally, so black it here
        # to simplify comparison.
        ws_connection.credentials.authority_url = ""
        self.check_rest_conversion_stable(ws_connection)

        # Note: also tests AAD credential
        ws_connection = load_connection(source="./tests/test_configs/connection/one_lake_with_id.yaml")
        assert type(ws_connection) == MicrosoftOneLakeConnection
        assert ws_connection.type == camel_to_snake(ConnectionCategory.AZURE_ONE_LAKE)
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.AAD)
        assert ws_connection.name == "one_lake_with_id"
        assert ws_connection.target == "https://www.endpoint.com/the_workspace_name/1234567-1234-1234-1234-123456789012"
        self.check_all_conversions_stable(ws_connection)

    def test_azure_open_ai(self):
        ws_connection = load_connection(source="./tests/test_configs/connection/azure_open_ai_api.yaml")
        assert type(ws_connection) == AzureOpenAIConnection
        assert ws_connection.type == camel_to_snake(ConnectionCategory.AZURE_OPEN_AI)
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.API_KEY)
        assert ws_connection.credentials.key == "12344"
        assert ws_connection.api_key == "12344"
        assert ws_connection.name == "test_ws_conn_open_ai"
        assert ws_connection.target == "dummy"
        assert ws_connection.metadata["hello"] == "world"
        assert ws_connection.metadata["ApiType"] == "Azure"
        assert ws_connection.api_version == "1.0"
        assert ws_connection.open_ai_resource_id == "some id"
        self.check_all_conversions_stable(ws_connection)

        ws_connection = load_connection(source="./tests/test_configs/connection/azure_open_ai_entra.yaml")
        assert type(ws_connection) == AzureOpenAIConnection
        assert ws_connection.type == camel_to_snake(ConnectionCategory.AZURE_OPEN_AI)
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.AAD)
        assert ws_connection.name == "azure_open_ai_conn_entra"
        assert ws_connection.target == "dummy"
        assert ws_connection.metadata["hello"] == "world"
        assert ws_connection.metadata["ApiType"] == "Azure"
        assert ws_connection.api_version is None
        assert ws_connection.open_ai_resource_id is None
        self.check_all_conversions_stable(ws_connection)

    def test_ai_services(self):
        ws_connection = load_connection(source="./tests/test_configs/connection/ai_services_with_key.yaml")
        assert type(ws_connection) == AzureAIServicesConnection
        assert ws_connection.type == ConnectionTypes.AZURE_AI_SERVICES
        assert ws_connection.api_key == "2222"
        assert ws_connection.name == "ai_services_conn_api"
        assert ws_connection.target == "my_endpoint"
        assert ws_connection.ai_services_resource_id == "this-needs-to-be-a-valid-ai-services-id-in-e2e-tests"
        self.check_all_conversions_stable(ws_connection)

        ws_connection = load_connection(source="./tests/test_configs/connection/ai_services_with_entra.yaml")
        assert type(ws_connection) == AzureAIServicesConnection
        assert ws_connection.type == ConnectionTypes.AZURE_AI_SERVICES
        assert ws_connection.name == "ai_services_conn_entra"
        assert ws_connection.target == "my_endpoint"
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.AAD)
        assert ws_connection.ai_services_resource_id == "this-needs-to-be-a-valid-ai-services-id-in-e2e-tests"
        self.check_all_conversions_stable(ws_connection)

    def test_content_safety(self):
        ws_connection = load_connection(source="./tests/test_configs/connection/content_safety_with_key.yaml")
        assert type(ws_connection) == AzureContentSafetyConnection
        assert ws_connection.type == ConnectionTypes.AZURE_CONTENT_SAFETY
        assert ws_connection.metadata["Kind"] == CognitiveServiceKinds.CONTENT_SAFETY
        assert ws_connection.api_key == "2222"
        assert ws_connection.name == "content_safety_conn_api"
        assert ws_connection.target == "my_endpoint"
        self.check_all_conversions_stable(ws_connection)

        ws_connection = load_connection(source="./tests/test_configs/connection/content_safety_with_entra.yaml")
        assert type(ws_connection) == AzureContentSafetyConnection
        assert ws_connection.type == ConnectionTypes.AZURE_CONTENT_SAFETY
        assert ws_connection.metadata["Kind"] == CognitiveServiceKinds.CONTENT_SAFETY
        assert ws_connection.name == "content_safety_conn_entra"
        assert ws_connection.target == "my_endpoint"
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.AAD)
        self.check_all_conversions_stable(ws_connection)

    def test_speech(self):
        ws_connection = load_connection(source="./tests/test_configs/connection/speech_with_key.yaml")

        assert type(ws_connection) == AzureSpeechServicesConnection
        assert ws_connection.type == ConnectionTypes.AZURE_SPEECH_SERVICES
        assert ws_connection.metadata["Kind"] == CognitiveServiceKinds.SPEECH
        assert ws_connection.api_key == "2222"
        assert ws_connection.name == "speech_api"
        assert ws_connection.target == "my_endpoint"
        self.check_all_conversions_stable(ws_connection)

        ws_connection = load_connection(source="./tests/test_configs/connection/speech_with_entra.yaml")
        assert type(ws_connection) == AzureSpeechServicesConnection
        assert ws_connection.type == ConnectionTypes.AZURE_SPEECH_SERVICES
        assert ws_connection.metadata["Kind"] == CognitiveServiceKinds.SPEECH
        assert ws_connection.name == "speech_entra"
        assert ws_connection.target == "my_endpoint"
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.AAD)
        self.check_all_conversions_stable(ws_connection)

    def test_search(self):
        ws_connection = load_connection(source="./tests/test_configs/connection/search_with_key.yaml")
        assert type(ws_connection) == AzureAISearchConnection
        assert ws_connection.type == ConnectionTypes.AZURE_SEARCH
        assert ws_connection.api_key == "3333"
        assert ws_connection.name == "search_api"
        assert ws_connection.target == "this_is_a_target"
        self.check_all_conversions_stable(ws_connection)

        ws_connection = load_connection(source="./tests/test_configs/connection/search_with_entra.yaml")
        assert type(ws_connection) == AzureAISearchConnection
        assert ws_connection.type == ConnectionTypes.AZURE_SEARCH
        assert ws_connection.name == "search_entra"
        assert ws_connection.target == "this_is_a_target_too"
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.AAD)
        self.check_all_conversions_stable(ws_connection)

    def test_api_key(self):
        ws_connection = load_connection(source="./tests/test_configs/connection/api_key_conn.yaml")
        assert type(ws_connection) == APIKeyConnection
        assert ws_connection.type == camel_to_snake(ConnectionCategory.API_KEY)
        assert ws_connection.api_key == "3232"
        assert ws_connection.name == "just_api"
        assert ws_connection.api_base == "this_is_a_target"
        self.check_all_conversions_stable(ws_connection)

    def test_custom(self):
        ws_connection = load_connection(source="./tests/test_configs/connection/custom.yaml")
        assert type(ws_connection) == WorkspaceConnection
        assert ws_connection.type == camel_to_snake(ConnectionTypes.CUSTOM)
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.API_KEY)
        assert ws_connection.credentials.key == "4444"
        assert ws_connection.name == "test_ws_conn_custom_keys"
        assert ws_connection.target == "my_endpoint"
        assert ws_connection.metadata["one"] == "two"
        self.check_all_conversions_stable(ws_connection)

    def test_open_ai(self):
        ws_connection = load_connection(source="./tests/test_configs/connection/not_azure_open_ai.yaml")
        assert type(ws_connection) == OpenAIConnection
        assert ws_connection.type == camel_to_snake(ConnectionCategory.OPEN_AI)
        assert ws_connection.api_key == "123446"
        assert ws_connection.name == "open_ai_conn"
        assert ws_connection.target is None
        self.check_all_conversions_stable(ws_connection)

    def test_serp(self):
        ws_connection = load_connection(source="./tests/test_configs/connection/serp.yaml")
        assert type(ws_connection) == SerpConnection
        assert ws_connection.type == camel_to_snake(ConnectionCategory.SERP)
        assert ws_connection.api_key == "1234467"
        assert ws_connection.name == "serp_conn"
        assert ws_connection.target is None
        self.check_all_conversions_stable(ws_connection)

    def test_git(self):
        ws_connection = load_connection(source="./tests/test_configs/connection/git_pat.yaml")

        assert type(ws_connection) == WorkspaceConnection
        assert ws_connection.type == camel_to_snake(ConnectionCategory.GIT)
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.PAT)
        assert ws_connection.credentials.pat == "dummy_pat"
        assert ws_connection.name == "test_ws_conn_git_pat"
        assert ws_connection.target == "https://test-git-feed.com"
        self.check_all_conversions_stable(ws_connection)

        ws_connection = load_connection(source="./tests/test_configs/connection/git_no_cred.yaml")
        assert type(ws_connection) == WorkspaceConnection
        assert ws_connection.type == camel_to_snake(ConnectionCategory.GIT)
        assert ws_connection.credentials.type == ConnectionAuthType.NONE
        assert ws_connection.name == "git_no_cred_conn"
        assert ws_connection.target == "https://test-git-feed.com2"
        self.check_all_conversions_stable(ws_connection)

    def test_python_feed(self):
        ws_connection = load_connection(source="./tests/test_configs/connection/python_feed_pat.yaml")
        assert type(ws_connection) == WorkspaceConnection
        assert ws_connection.type == camel_to_snake(ConnectionCategory.PYTHON_FEED)
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.PAT)
        assert ws_connection.credentials.pat == "dummy_pat"
        assert ws_connection.name == "test_ws_conn_python_pat"
        assert ws_connection.target == "https://test-feed.com"
        self.check_all_conversions_stable(ws_connection)

        ws_connection = load_connection(source="./tests/test_configs/connection/python_feed_user_pass.yaml")
        assert type(ws_connection) == WorkspaceConnection
        assert ws_connection.type == camel_to_snake(ConnectionCategory.PYTHON_FEED)
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.USERNAME_PASSWORD)
        assert ws_connection.credentials.username == "john"
        assert ws_connection.credentials.password == "halo"
        assert ws_connection.name == "test_ws_conn_python_user_pass"
        assert ws_connection.target == "https://test-feed.com2"
        self.check_all_conversions_stable(ws_connection)

        ws_connection = load_connection(source="./tests/test_configs/connection/python_feed_no_cred.yaml")
        assert type(ws_connection) == WorkspaceConnection
        assert ws_connection.type == camel_to_snake(ConnectionCategory.PYTHON_FEED)
        assert ws_connection.credentials.type == ConnectionAuthType.NONE
        assert ws_connection.name == "test_ws_conn_python_no_cred"
        assert ws_connection.target == "https://test-feed.com3"
        self.check_all_conversions_stable(ws_connection)

    def test_container_registry(self):
        ws_connection = load_connection(
            source="./tests/test_configs/connection/container_registry_managed_identity.yaml"
        )
        assert ws_connection.type == camel_to_snake(ConnectionCategory.CONTAINER_REGISTRY)
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.MANAGED_IDENTITY)
        assert ws_connection.credentials.client_id == "client_id"
        assert ws_connection.credentials.resource_id == "resource_id"
        assert ws_connection.name == "test_ws_conn_cr_managed"
        assert ws_connection.target == "https://test-feed.com"
        self.check_all_conversions_stable(ws_connection)

        ws_connection = load_connection(source="./tests/test_configs/connection/container_registry_user_pass.yaml")
        assert ws_connection.type == camel_to_snake(ConnectionCategory.CONTAINER_REGISTRY)
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.USERNAME_PASSWORD)
        assert ws_connection.credentials.username == "springer"
        assert ws_connection.credentials.password == "spaniel"
        assert ws_connection.name == "test_ws_conn_cr_user_pass"
        assert ws_connection.target == "https://test-feed.com2"
        self.check_all_conversions_stable(ws_connection)

    def test_serverless(self):
        ws_connection = load_connection(source="./tests/test_configs/connection/serverless_api.yaml")

        assert type(ws_connection) == ServerlessConnection
        assert ws_connection.type == camel_to_snake(ConnectionCategory.SERVERLESS)
        assert ws_connection.api_key == "1029"
        assert ws_connection.name == "serverless_with_api"
        assert ws_connection.target == "serverless_endpoint"
        self.check_all_conversions_stable(ws_connection)

    # Old connection tests. These have not had their valid credential inputs verified by any known recent spec.

    def test_s3_access_key(self):
        ws_connection = load_connection(source="./tests/test_configs/connection/s3_access_key.yaml")

        assert ws_connection.type == camel_to_snake(ConnectionCategory.S3)
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.ACCESS_KEY)
        assert ws_connection.credentials.access_key_id == "dummy"
        assert ws_connection.credentials.secret_access_key == "dummy"
        assert ws_connection.name == "test_ws_conn_s3"
        assert ws_connection.target == "dummy"
        assert ws_connection.metadata == {}

    def test_snowflake(self):
        ws_connection = load_connection(source="./tests/test_configs/connection/snowflake_user_pwd.yaml")

        assert ws_connection.type == camel_to_snake(ConnectionCategory.SNOWFLAKE)
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.USERNAME_PASSWORD)
        assert ws_connection.credentials.username == "dummy"
        assert ws_connection.credentials.password == "dummy"
        assert ws_connection.name == "test_ws_conn_snowflake"
        assert ws_connection.target == "dummy"
        assert ws_connection.metadata == {}

    def test_azure_sql_db(self):
        ws_connection = load_connection(source="./tests/test_configs/connection/azure_sql_db_user_pwd.yaml")

        assert ws_connection.type == camel_to_snake(ConnectionCategory.AZURE_SQL_DB)
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.USERNAME_PASSWORD)
        assert ws_connection.credentials.username == "dummy"
        assert ws_connection.credentials.password == "dummy"
        assert ws_connection.name == "test_ws_conn_azure_sql_db"
        assert ws_connection.target == "dummy"
        assert ws_connection.metadata == {}

    def test_synapse_analytics(self):
        ws_connection = load_connection(source="./tests/test_configs/connection/azure_synapse_analytics_user_pwd.yaml")

        assert ws_connection.type == camel_to_snake(ConnectionCategory.AZURE_SYNAPSE_ANALYTICS)
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.USERNAME_PASSWORD)
        assert ws_connection.credentials.username == "dummy"
        assert ws_connection.credentials.password == "dummy"
        assert ws_connection.name == "test_ws_conn_azure_synapse_analytics"
        assert ws_connection.target == "dummy"
        assert ws_connection.metadata == {}

    def test_my_sql_db(self):
        ws_connection = load_connection(source="./tests/test_configs/connection/azure_my_sql_db_user_pwd.yaml")

        assert ws_connection.type == camel_to_snake(ConnectionCategory.AZURE_MY_SQL_DB)
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.USERNAME_PASSWORD)
        assert ws_connection.credentials.username == "dummy"
        assert ws_connection.credentials.password == "dummy"
        assert ws_connection.name == "test_ws_conn_azure_my_sql_db"
        assert ws_connection.target == "dummy"
        assert ws_connection.metadata == {}

    def test_postgres_db(self):
        ws_connection = load_connection(source="./tests/test_configs/connection/azure_postgres_db_user_pwd.yaml")

        assert ws_connection.type == camel_to_snake(ConnectionCategory.AZURE_POSTGRES_DB)
        assert ws_connection.credentials.type == camel_to_snake(ConnectionAuthType.USERNAME_PASSWORD)
        assert ws_connection.credentials.username == "dummy"
        assert ws_connection.credentials.password == "dummy"
        assert ws_connection.name == "test_ws_conn_azure_postgres_db"
        assert ws_connection.target == "dummy"
        assert ws_connection.metadata == {}

    def empty_credentials_rest_conversion(self):
        ws_connection = WorkspaceConnection(
            target="dummy_target",
            type=camel_to_snake(ConnectionCategory.PYTHON_FEED),
            credentials=None,
            name="dummy_connection",
            metadata=None,
        )
        rest_conn = ws_connection._to_rest_object()
        new_ws_conn = AzureOpenAIConnection._from_rest_object(rest_obj=rest_conn)
        assert new_ws_conn.credentials == None

    def test_ws_conn_subtype_restriction(self):
        with pytest.raises(ValueError):
            _ = WorkspaceConnection(
                target="dummy_target",
                type=ConnectionCategory.AZURE_OPEN_AI,
                name="dummy_connection",
                strict_typing=True,
                credentials=None,
            )
        _ = AzureOpenAIConnection(
            azure_endpoint="dummy_target",
            name="dummy_connection",
            strict_typing=True,
            credentials=None,
        )

    def test_bad_credential_load_attempts(self):
        with pytest.raises(ValidationError):
            _ = load_connection(source="./tests/test_configs/connection/blob_store_bad_cred.yaml")

        with pytest.raises(ValueError):
            _ = WorkspaceConnection(
                name="bad",
                target="bad",
                type=ConnectionTypes.AZURE_DATA_LAKE_GEN_2,
                credentials=ApiKeyConfiguration(key="bad"),
            )



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from typing import Any, Optional

from azure.ai.ml._restclient.v2023_04_01_preview.models import StackEnsembleSettings as RestStackEnsembleSettings
from azure.ai.ml._restclient.v2023_04_01_preview.models import StackMetaLearnerType
from azure.ai.ml.entities._mixins import RestTranslatableMixin


class StackEnsembleSettings(RestTranslatableMixin):
    """Advance setting to customize StackEnsemble run."""

    def __init__(
        self,
        *,
        stack_meta_learner_k_wargs: Optional[Any] = None,
        stack_meta_learner_train_percentage: float = 0.2,
        stack_meta_learner_type: Optional[StackMetaLearnerType] = None,
        **kwargs: Any
    ):
        """
        :param stack_meta_learner_k_wargs: Optional parameters to pass to the initializer of the
         meta-learner.
        :type stack_meta_learner_k_wargs: any
        :param stack_meta_learner_train_percentage: Specifies the proportion of the training set
         (when choosing train and validation type of training) to be reserved for training the
         meta-learner. Default value is 0.2.
        :type stack_meta_learner_train_percentage: float
        :param stack_meta_learner_type: The meta-learner is a model trained on the output of the
         individual heterogeneous models. Possible values include: "None", "LogisticRegression",
         "LogisticRegressionCV", "LightGBMClassifier", "ElasticNet", "ElasticNetCV",
         "LightGBMRegressor", "LinearRegression".
        :type stack_meta_learner_type: str or
         ~azure.mgmt.machinelearningservices.models.StackMetaLearnerType
        """
        super(StackEnsembleSettings, self).__init__(**kwargs)
        self.stack_meta_learner_k_wargs = stack_meta_learner_k_wargs
        self.stack_meta_learner_train_percentage = stack_meta_learner_train_percentage
        self.stack_meta_learner_type = stack_meta_learner_type

    def _to_rest_object(self) -> RestStackEnsembleSettings:
        return RestStackEnsembleSettings(
            stack_meta_learner_k_wargs=self.stack_meta_learner_k_wargs,
            stack_meta_learner_train_percentage=self.stack_meta_learner_train_percentage,
            stack_meta_learner_type=self.stack_meta_learner_type,
        )

    @classmethod
    def _from_rest_object(cls, obj: RestStackEnsembleSettings) -> "StackEnsembleSettings":
        return cls(
            stack_meta_learner_k_wargs=obj.stack_meta_learner_k_wargs,
            stack_meta_learner_train_percentage=obj.stack_meta_learner_train_percentage,
            stack_meta_learner_type=obj.stack_meta_learner_type,
        )

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, StackEnsembleSettings):
            return NotImplemented

        return (
            super().__eq__(other)
            and self.stack_meta_learner_k_wargs == other.stack_meta_learner_k_wargs
            and self.stack_meta_learner_train_percentage == other.stack_meta_learner_train_percentage
            and self.stack_meta_learner_type == other.stack_meta_learner_type
        )

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

import json
import logging
from typing import Any, Dict, Optional

from azure.ai.ml._restclient.v2023_04_01_preview.models import ResourceConfiguration as RestResourceConfiguration
from azure.ai.ml.constants._job.job import JobComputePropertyFields
from azure.ai.ml.entities._mixins import DictMixin, RestTranslatableMixin

module_logger = logging.getLogger(__name__)


class ResourceConfiguration(RestTranslatableMixin, DictMixin):
    """Resource configuration for a job.

    This class should not be instantiated directly. Instead, use its subclasses.

    :keyword instance_count: The number of instances to use for the job.
    :paramtype instance_count: Optional[int]
    :keyword instance_type: The type of instance to use for the job.
    :paramtype instance_type: Optional[str]
    :keyword properties: The resource's property dictionary.
    :paramtype properties: Optional[dict[str, Any]]
    """

    def __init__(
        self,  # pylint: disable=unused-argument
        *,
        instance_count: Optional[int] = None,
        instance_type: Optional[str] = None,
        properties: Optional[Dict[str, Any]] = None,
        **kwargs: Any
    ) -> None:
        self.instance_count = instance_count
        self.instance_type = instance_type
        self.properties = {}
        if properties is not None:
            for key, value in properties.items():
                if key == JobComputePropertyFields.AISUPERCOMPUTER:
                    self.properties[JobComputePropertyFields.SINGULARITY.lower()] = value
                else:
                    self.properties[key] = value

    def _to_rest_object(self) -> RestResourceConfiguration:
        serialized_properties = {}
        if self.properties:
            for key, value in self.properties.items():
                try:
                    if (
                        key.lower() == JobComputePropertyFields.SINGULARITY.lower()
                        or key.lower() == JobComputePropertyFields.AISUPERCOMPUTER.lower()
                    ):
                        # Map Singularity -> AISupercomputer in SDK until MFE does mapping
                        key = JobComputePropertyFields.AISUPERCOMPUTER
                    # recursively convert Ordered Dict to dictionary
                    serialized_properties[key] = json.loads(json.dumps(value))
                except Exception:  # pylint: disable=W0718
                    pass
        return RestResourceConfiguration(
            instance_count=self.instance_count,
            instance_type=self.instance_type,
            properties=serialized_properties,
        )

    @classmethod
    def _from_rest_object(  # pylint: disable=arguments-renamed
        cls, rest_obj: Optional[RestResourceConfiguration]
    ) -> Optional["ResourceConfiguration"]:
        if rest_obj is None:
            return None
        return ResourceConfiguration(
            instance_count=rest_obj.instance_count,
            instance_type=rest_obj.instance_type,
            properties=rest_obj.properties,
            deserialize_properties=True,
        )

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, ResourceConfiguration):
            return NotImplemented
        return self.instance_count == other.instance_count and self.instance_type == other.instance_type

    def __ne__(self, other: object) -> bool:
        if not isinstance(other, ResourceConfiguration):
            return NotImplemented
        return not self.__eq__(other)

    def _merge_with(self, other: "ResourceConfiguration") -> None:
        if other:
            if other.instance_count:
                self.instance_count = other.instance_count
            if other.instance_type:
                self.instance_type = other.instance_type
            if other.properties:
                self.properties = other.properties



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------


from typing import Dict, Union

from azure.ai.ml.entities._inputs_outputs import Input, Output
from azure.ai.ml.entities._job._input_output_helpers import build_input_output


class JobIOMixin:
    @property
    def inputs(self) -> Dict[str, Union[Input, str, bool, int, float]]:
        return self._inputs

    @inputs.setter
    def inputs(self, value: Dict[str, Union[Input, str, bool, int, float]]) -> None:
        self._inputs: Dict = {}
        if not value:
            return

        for input_name, input_value in value.items():
            self._inputs[input_name] = build_input_output(input_value)

    @property
    def outputs(self) -> Dict[str, Output]:
        return self._outputs

    @outputs.setter
    def outputs(self, value: Dict[str, Output]) -> None:
        self._outputs: Dict = {}
        if not value:
            return

        for output_name, output_value in value.items():
            self._outputs[output_name] = build_input_output(output_value, inputs=False)



import pytest
from marshmallow import ValidationError

from azure.ai.ml import load_job
from azure.ai.ml.entities import PipelineJob
from azure.ai.ml.entities._job.to_rest_functions import to_rest_job_object
from azure.ai.ml.exceptions import ValidationException

from .._util import _PIPELINE_JOB_TIMEOUT_SECOND


@pytest.mark.timeout(_PIPELINE_JOB_TIMEOUT_SECOND)
@pytest.mark.usefixtures(
    "enable_pipeline_private_preview_features",
    "enable_private_preview_schema_features",
    "enable_private_preview_pipeline_node_types",
)
@pytest.mark.unittest
@pytest.mark.pipeline_test
class TestControlFlowPipelineJobUT:
    pass


class TestIfElseUI(TestControlFlowPipelineJobUT):
    @pytest.mark.parametrize(
        "exception_cls, yaml_path, msg, location",
        [
            # None true & None false
            (
                ValidationError,
                "./tests/test_configs/pipeline_jobs/invalid/if_else/none_true_none_false.yml",
                "True block and false block cannot be empty at the same time.",
                "",
            ),
            # None true & empty false
            (
                ValidationError,
                "./tests/test_configs/pipeline_jobs/invalid/if_else/none_true_empty_false.yml",
                "True block and false block cannot be empty at the same time.",
                "",
            ),
            # true & false intersection
            (
                ValidationError,
                "./tests/test_configs/pipeline_jobs/invalid/if_else/true_false_intersection.yml",
                "True block and false block cannot contain same nodes:",
                "",
            ),
            # invalid binding
            (
                ValidationException,
                "./tests/test_configs/pipeline_jobs/invalid/if_else/invalid_binding.yml",
                "of dsl.condition has invalid binding expression",
                '"path": "jobs.conditionnode.true_block",',
            ),
        ],
    )
    def test_if_else_validate(self, exception_cls, yaml_path, msg, location):
        with pytest.raises(exception_cls) as e:
            job = load_job(yaml_path)
            job._validate(raise_error=True)

        assert msg in str(e.value)
        assert location in str(e.value)


class TestDoWhilePipelineJobUT(TestControlFlowPipelineJobUT):
    def test_do_while_true_pipeline_omit_condition(self):
        yaml_path = "./tests/test_configs/pipeline_jobs/control_flow/do_while/pipeline.yml"
        pipeline_job = load_job(yaml_path)
        rest_job_resource = to_rest_job_object(pipeline_job)
        assert "condition" in rest_job_resource.properties.jobs["do_while_job_with_pipeline_job"]
        assert "condition" not in rest_job_resource.properties.jobs["do_while_true_job_with_pipeline_job"]

    def test_do_while_pipeline_illegal_cases(self):
        yaml_path = "./tests/test_configs/pipeline_jobs/control_flow/do_while/invalid_pipeline.yml"
        expected_validation_result = [
            # bool type is illegal for field condition
            (
                "Not a valid string.; Not a valid string.",
                "jobs.invalid_condition.condition",
            ),
            (
                "Missing data for required field.",
                "jobs.empty_mapping.mapping",
            ),
            (
                "Must be greater than or equal to 1 and less than or equal to 1000.",
                "jobs.out_of_range_max_iteration_count.limits.max_iteration_count",
            ),
            (
                "Missing data for required field.",
                "jobs.empty_max_iteration_count.limits",
            ),
            (
                "Not a valid integer.",
                "jobs.invalid_max_iteration_count.limits.max_iteration_count",
            ),
        ]
        with pytest.raises(ValidationError) as e:
            load_job(yaml_path)
        error_message = str(e.value)
        # use count of "invalid_pipeline.yml#line" to get number of error messages
        assert error_message.count("invalid_pipeline.yml#line") == len(expected_validation_result)
        for msg, location in expected_validation_result:
            assert msg in error_message
            assert location in error_message


class TestParallelForPipelineJobUT(TestControlFlowPipelineJobUT):
    @pytest.mark.parametrize(
        "exception_cls, yaml_path, msg, location",
        [
            pytest.param(
                ValidationError,
                "./tests/test_configs/pipeline_jobs/invalid/parallel_for/items_invalid_value_type.yml",
                "Not a valid mapping type.",
                '"path": "jobs.parallelfor.items",',
                id="items_invalid_content_type",
            ),
            pytest.param(
                ValidationException,
                "./tests/test_configs/pipeline_jobs/invalid/parallel_for/items_empty.yml",
                "Items is an empty list/dict.",
                '"path": "jobs.parallelfor.items",',
                id="items_empty_dict_content",
            ),
            pytest.param(
                ValidationException,
                "./tests/test_configs/pipeline_jobs/invalid/parallel_for/items_meta_mismatch.yml",
                '"message": "Items should have same keys',
                '"path": "jobs.parallelfor.items"',
                id="items_meta_mismatch",
            ),
            pytest.param(
                ValidationException,
                "./tests/test_configs/pipeline_jobs/invalid/parallel_for/items_not_exist.yml",
                "got unmatched inputs with loop body component",
                '"path": "jobs.parallelfor.items"',
                id="items_not_exist",
            ),
            pytest.param(
                ValidationException,
                "./tests/test_configs/pipeline_jobs/invalid/parallel_for/items_invalid_json.yml",
                '"message": "Items is neither a valid JSON',
                '"path": "jobs.parallelfor.items"',
                id="items_invalid_json",
            ),
            pytest.param(
                ValidationError,
                "./tests/test_configs/pipeline_jobs/invalid/parallel_for/items_unprovided.yml",
                '"message": "Missing data for required field',
                "items_unprovided.yml#line 7",
                id="required_field_unprovided",
            ),
            pytest.param(
                ValidationException,
                "./tests/test_configs/pipeline_jobs/invalid/parallel_for/body_not_supported.yml",
                " got <class 'azure.ai.ml.entities._builders.parallel.Parallel'> instead.",
                "",
                id="body_not_supported",
            ),
        ],
    )
    def test_dsl_parallel_for_pipeline_illegal_cases(self, exception_cls, yaml_path, msg, location):
        with pytest.raises(exception_cls) as e:
            job = load_job(yaml_path)
            job._validate(raise_error=True)

        assert msg in str(e.value)
        assert location in str(e.value)



import copy
import json
import os
from pathlib import Path
from unittest import mock

import pydash
import pytest
import yaml

from azure.ai.ml import MLClient, load_component
from azure.ai.ml._restclient.v2022_05_01.models import ComponentVersionData
from azure.ai.ml._utils._arm_id_utils import PROVIDER_RESOURCE_ID_WITH_VERSION
from azure.ai.ml.constants._common import BASE_PATH_CONTEXT_KEY, PARAMS_OVERRIDE_KEY, AssetTypes, LegacyAssetTypes
from azure.ai.ml.constants._component import ComponentSource
from azure.ai.ml.entities import CommandComponent, Component, PipelineComponent
from azure.ai.ml.entities._assets import Code
from azure.ai.ml.entities._component.component import COMPONENT_PLACEHOLDER
from azure.ai.ml.entities._component.component_factory import component_factory

from .._util import _COMPONENT_TIMEOUT_SECOND

tests_root_dir = Path(__file__).parent.parent.parent
components_dir = tests_root_dir / "test_configs/components/"


def load_component_entity_from_yaml(
    path: str,
    mock_machinelearning_client: MLClient,
    context={},
    is_anonymous=False,
    fields_to_override=None,
    _type="command",
) -> CommandComponent:
    """Component yaml -> component entity -> rest component object -> component entity"""
    with open(path, "r") as f:
        data = yaml.safe_load(f)
    context.update({BASE_PATH_CONTEXT_KEY: Path(path).parent})
    create_instance_func, create_schema_func = component_factory.get_create_funcs(data)
    data = dict(create_schema_func(context).load(data))
    if fields_to_override is None:
        fields_to_override = {}
    data.update(fields_to_override)
    if is_anonymous is True:
        data["name"] = None
        data["version"] = None

    internal_representation = create_instance_func()
    internal_representation.__init__(
        **data,
    )
    internal_representation._base_path = context[BASE_PATH_CONTEXT_KEY]

    def mock_get_asset_arm_id(*args, **kwargs):
        if len(args) > 0:
            arg = args[0]
            if isinstance(arg, str):
                return arg
            elif isinstance(arg, Code):
                if COMPONENT_PLACEHOLDER in str(arg.path):
                    # for generated code, return content in it
                    with open(arg.path) as f:
                        return f.read()
                elif os.path.isfile(arg.path):
                    return f"{str(arg.path)}:1"
                else:
                    return f"{Path(arg.path).name}:1"
        return "xxx"

    # change internal assets into arm id
    with mock.patch(
        "azure.ai.ml.operations._operation_orchestrator.OperationOrchestrator.get_asset_arm_id",
        side_effect=mock_get_asset_arm_id,
    ):
        mock_machinelearning_client.components._resolve_arm_id_or_upload_dependencies(internal_representation)
    rest_component = internal_representation._to_rest_object()
    # set arm id before deserialize
    mock_workspace_scope = mock_machinelearning_client._operation_scope
    rest_component.id = PROVIDER_RESOURCE_ID_WITH_VERSION.format(
        mock_workspace_scope.subscription_id,
        mock_workspace_scope.resource_group_name,
        mock_workspace_scope.workspace_name,
        "components",
        internal_representation.name,
        "1",
    )
    internal_component = CommandComponent._from_rest_object(rest_component)
    return internal_component


def load_component_entity_from_rest_json(path) -> Component:
    """Rest component json -> rest component object -> component entity"""
    with open(path, "r") as f:
        target = yaml.safe_load(f)
    rest_obj = ComponentVersionData.from_dict(json.loads(json.dumps(target)))
    internal_component = Component._from_rest_object(rest_obj)
    return internal_component


@pytest.mark.timeout(_COMPONENT_TIMEOUT_SECOND)
@pytest.mark.unittest
@pytest.mark.pipeline_test
class TestCommandComponent:
    def test_serialize_deserialize_basic(self, mock_machinelearning_client: MLClient):
        test_path = "./tests/test_configs/components/helloworld_component.yml"
        component_entity = load_component_entity_from_yaml(test_path, mock_machinelearning_client)
        rest_path = "./tests/test_configs/components/helloworld_component_rest.json"
        target_entity = load_component_entity_from_rest_json(rest_path)

        # skip check code and environment
        component_dict = component_entity._to_dict()
        assert component_dict["id"]
        component_dict = pydash.omit(dict(component_dict), "command", "environment", "code", "id")
        expected_dict = pydash.omit(
            dict(target_entity._to_dict()),
            "command",
            "environment",
            "code",
            "creation_context",
            "id",
        )
        assert component_dict == expected_dict
        assert component_entity.code is None
        assert component_entity.environment == "AzureML-sklearn-1.0-ubuntu20.04-py38-cpu:33"

    def test_serialize_deserialize_environment_no_version(self, mock_machinelearning_client: MLClient):
        test_path = "./tests/test_configs/components/helloworld_component_alt1.yml"
        component_entity = load_component_entity_from_yaml(test_path, mock_machinelearning_client)

        assert component_entity.environment == "AzureML-sklearn-1.0-ubuntu20.04-py38-cpu:33"

    def test_serialize_deserialize_input_types(self, mock_machinelearning_client: MLClient):
        test_path = "./tests/test_configs/components/input_types_component.yml"
        component_entity = load_component_entity_from_yaml(test_path, mock_machinelearning_client)
        rest_path = "./tests/test_configs/components/input_types_component_rest.json"
        target_entity = load_component_entity_from_rest_json(rest_path)

        # skip check code and environment
        component_dict = pydash.omit(dict(component_entity._to_dict()), "command", "environment", "code", "id")
        expected_dict = pydash.omit(
            dict(target_entity._to_dict()),
            "command",
            "environment",
            "creation_context",
            "code",
            "id",
        )
        assert component_dict == expected_dict

        rest_component_resource = component_entity._to_rest_object()
        assert rest_component_resource.properties.component_spec["inputs"] == expected_dict["inputs"]

    def test_override_params(self, mock_machinelearning_client: MLClient):
        test_path = "./tests/test_configs/components/helloworld_component.yml"
        context = {
            PARAMS_OVERRIDE_KEY: [
                {
                    "inputs.component_in_path": {
                        "description": "override component_in_path",
                        "type": "uri_folder",
                    }
                }
            ]
        }
        component_entity = load_component_entity_from_yaml(test_path, mock_machinelearning_client, context)
        inputs_dict = {k: v._to_dict() for k, v in component_entity.inputs.items()}
        assert inputs_dict == {
            "component_in_number": {
                "type": "number",
                "default": 10.99,
                "description": "A number",
                "optional": True,
            },
            "component_in_path": {
                "type": "uri_folder",
                "description": "override component_in_path",
            },
        }

        override_inputs = {
            # according to InputPortSchema, mode in component input will not take effect
            "component_in_path": {"type": "uri_folder"},
            "component_in_number": {"max": 1.0, "min": 0.0, "type": "number"},
            "override_param3": {"optional": True, "type": "integer"},
            "override_param4": {"default": False, "type": "boolean"},
            "override_param5": {"default": "str", "type": "string"},
            "override_param6": {"enum": ["enum1", "enum2", "enum3"], "type": "string"},
        }
        context = {PARAMS_OVERRIDE_KEY: [{"inputs": copy.deepcopy(override_inputs)}]}
        component_entity = load_component_entity_from_yaml(test_path, mock_machinelearning_client, context)
        inputs_dict = {k: v._to_dict() for k, v in component_entity.inputs.items()}
        assert inputs_dict == override_inputs

    def test_serialize_deserialize_with_code_path(self, mock_machinelearning_client: MLClient):
        test_path = "./tests/test_configs/components/basic_component_code_local_path.yml"
        component_entity = load_component_entity_from_yaml(test_path, mock_machinelearning_client)
        # make sure code has "created"
        assert component_entity.code

    def test_serialize_deserialize_default_code(self, mock_machinelearning_client: MLClient):
        test_path = "./tests/test_configs/components/helloworld_component.yml"
        component_entity = load_component_entity_from_yaml(test_path, mock_machinelearning_client)
        assert component_entity.code is None

    def test_serialize_deserialize_input_output_path(self, mock_machinelearning_client: MLClient):
        expected_value_dict = {
            "path": {
                "inputs.component_in_path.type": LegacyAssetTypes.PATH,
                "inputs.component_in_file.type": AssetTypes.URI_FILE,
                "inputs.component_in_folder.type": AssetTypes.URI_FOLDER,
                "outputs.component_out_file.type": AssetTypes.URI_FILE,
                "outputs.component_out_folder.type": AssetTypes.URI_FOLDER,
            },
            "mltable": {
                "inputs.component_in_mltable_mount.type": AssetTypes.MLTABLE,
                "outputs.component_out_mltable_rw_mount.type": AssetTypes.MLTABLE,
            },
            "mlflow_model": {
                "inputs.component_in_mlflow_model_azure.type": AssetTypes.MLFLOW_MODEL,
                "outputs.component_out_mlflow_model.type": AssetTypes.MLFLOW_MODEL,
            },
            "custom_model": {
                "inputs.component_in_custom_model.type": AssetTypes.CUSTOM_MODEL,
                "outputs.component_out_custom_model.type": AssetTypes.CUSTOM_MODEL,
                "inputs.component_in_trition_model.type": AssetTypes.TRITON_MODEL,
                "outputs.component_out_trition_model.type": AssetTypes.TRITON_MODEL,
            },
        }
        for contract_type, expected_values in expected_value_dict.items():
            test_path = "./tests/test_configs/components/type_contract/{}.yml".format(contract_type)
            component_entity = load_component_entity_from_yaml(test_path, mock_machinelearning_client)
            component_entity_dict = component_entity._to_dict()
            for dot_key, expected_value in expected_values.items():
                assert pydash.get(component_entity_dict, dot_key) == expected_value

    def test_command_component_str(self):
        test_path = "./tests/test_configs/components/helloworld_component.yml"
        component_entity = load_component(source=test_path)
        component_str = str(component_entity)
        assert component_entity.name in component_str
        assert component_entity.version in component_str

    def test_anonymous_component_same_name(self, mock_machinelearning_client: MLClient):
        # scenario 1: same component interface, same code
        test_path1 = "./tests/test_configs/components/basic_component_code_local_path.yml"
        component_entity1 = load_component_entity_from_yaml(test_path1, mock_machinelearning_client, is_anonymous=True)
        component_hash1 = component_entity1._get_anonymous_hash()
        component_entity2 = load_component_entity_from_yaml(test_path1, mock_machinelearning_client, is_anonymous=True)
        component_hash2 = component_entity2._get_anonymous_hash()
        assert component_hash1 == component_hash2

        # scenario 2: same component, no code
        test_path2 = "./tests/test_configs/components/helloworld_component.yml"
        component_entity1 = load_component_entity_from_yaml(test_path2, mock_machinelearning_client, is_anonymous=True)
        component_hash1 = component_entity1._get_anonymous_hash()
        component_entity2 = load_component_entity_from_yaml(test_path2, mock_machinelearning_client, is_anonymous=True)
        component_hash2 = component_entity2._get_anonymous_hash()
        assert component_hash1 == component_hash2

        # scenario 3: same component interface, different code
        code_path1 = "./tests/test_configs/components/basic_component_code_local_path.yml"
        data1 = {"code": code_path1}
        code_path2 = "./tests/test_configs/components/helloworld_component.yml"
        data2 = {"code": code_path2}
        # only code is different in data1 and data2
        component_entity1 = load_component_entity_from_yaml(
            test_path1,
            mock_machinelearning_client,
            is_anonymous=True,
            fields_to_override=data1,
        )
        component_hash1 = component_entity1._get_anonymous_hash()
        component_entity2 = load_component_entity_from_yaml(
            test_path1,
            mock_machinelearning_client,
            is_anonymous=True,
            fields_to_override=data2,
        )
        component_hash2 = component_entity2._get_anonymous_hash()
        assert component_hash1 != component_hash2

        # scenario 4: different component interface, same code
        data1 = {"display_name": "CommandComponentBasic1"}
        data2 = {"display_name": "CommandComponentBasic2"}
        # only display name is different in data1 and data2
        component_entity1 = load_component_entity_from_yaml(
            test_path1,
            mock_machinelearning_client,
            is_anonymous=True,
            fields_to_override=data1,
        )
        component_hash1 = component_entity1._get_anonymous_hash()
        component_entity2 = load_component_entity_from_yaml(
            test_path1,
            mock_machinelearning_client,
            is_anonymous=True,
            fields_to_override=data2,
        )
        component_hash2 = component_entity2._get_anonymous_hash()
        assert component_hash1 != component_hash2

    def test_command_component_with_properties(self):
        test_path = "./tests/test_configs/components/helloworld_component_with_properties.yml"
        component_entity = load_component(source=test_path)
        assert component_entity.properties == {"azureml.pipelines.dynamic": "true"}

        validation_result = component_entity._validate()
        assert validation_result.passed is True

    def test_component_factory(self):
        test_path = "./tests/test_configs/components/helloworld_component_with_properties.yml"
        component_entity = load_component(source=test_path)
        recreated_component = component_factory.load_from_dict(
            data=component_entity._to_dict(),
            context={
                "source_path": test_path,
            },
            _source=ComponentSource.YAML_COMPONENT,
        )
        assert recreated_component._to_dict() == component_entity._to_dict()

        recreated_component = component_factory.load_from_rest(obj=component_entity._to_rest_object())
        assert recreated_component._to_dict() == component_entity._to_dict()

    def test_dump_with_non_existent_base_path(self):
        test_path = "./tests/test_configs/components/helloworld_component.yml"
        component_entity = load_component(source=test_path)
        component_entity._base_path = "/non/existent/path"
        component_entity._to_dict()

    def test_arm_code_from_rest_object(self):
        arm_code = (
            "azureml:/subscriptions/xxx/resourceGroups/xxx/providers/Microsoft.MachineLearningServices/"
            "workspaces/zzz/codes/90b33c11-365d-4ee4-aaa1-224a042deb41/versions/1"
        )
        yaml_path = "./tests/test_configs/components/helloworld_component.yml"
        yaml_component = load_component(yaml_path)

        from azure.ai.ml.entities import Component

        rest_object = yaml_component._to_rest_object()
        rest_object.properties.component_spec["code"] = arm_code
        component = Component._from_rest_object(rest_object)
        assert component.code == arm_code[8:]


@pytest.mark.timeout(_COMPONENT_TIMEOUT_SECOND)
@pytest.mark.unittest
class TestPipelineComponent:
    def test_inline_helloworld_pipeline_component(self) -> None:
        component_path = "./tests/test_configs/components/helloworld_inline_pipeline_component.yml"
        component: PipelineComponent = load_component(source=component_path)

        validation_result = component._validate()
        assert validation_result.passed is True

    def test_helloworld_pipeline_component(self) -> None:
        component_path = "./tests/test_configs/components/helloworld_pipeline_component.yml"
        component: PipelineComponent = load_component(source=component_path)

        validation_result = component._validate()
        assert validation_result.passed is True

    def test_helloworld_nested_pipeline_component(self) -> None:
        component_path = "./tests/test_configs/components/helloworld_nested_pipeline_component.yml"
        component: PipelineComponent = load_component(source=component_path)

        validation_result = component._validate()
        assert validation_result.passed is True


@pytest.mark.timeout(_COMPONENT_TIMEOUT_SECOND)
@pytest.mark.unittest
class TestSparkComponent:
    def test_component_load(self):
        # code is specified in yaml, value is respected
        component_yaml = "./tests/test_configs/dsl_pipeline/spark_job_in_pipeline/add_greeting_column_component.yml"
        spark_component = load_component(
            component_yaml,
        )
        validation_result = spark_component._validate()
        assert validation_result.passed is True



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import functools
from typing import TYPE_CHECKING
import warnings

from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.paging import ItemPaged
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import HttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.mgmt.core.exceptions import ARMErrorFormat
from msrest import Serializer

from .. import models as _models
from .._vendor import _convert_request, _format_url_section

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from typing import Any, Callable, Dict, Generic, Iterable, List, Optional, TypeVar
    T = TypeVar('T')
    ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False
# fmt: off

def build_get_dataset_definition_request(
    subscription_id,  # type: str
    resource_group_name,  # type: str
    workspace_name,  # type: str
    dataset_id,  # type: str
    version,  # type: str
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/dataset/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/datasets/{datasetId}/definitions/{version}')
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "workspaceName": _SERIALIZER.url("workspace_name", workspace_name, 'str'),
        "datasetId": _SERIALIZER.url("dataset_id", dataset_id, 'str'),
        "version": _SERIALIZER.url("version", version, 'str'),
    }

    url = _format_url_section(url, **path_format_arguments)

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="GET",
        url=url,
        headers=header_parameters,
        **kwargs
    )


def build_get_all_dataset_definitions_request(
    subscription_id,  # type: str
    resource_group_name,  # type: str
    workspace_name,  # type: str
    dataset_id,  # type: str
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    continuation_token_parameter = kwargs.pop('continuation_token_parameter', None)  # type: Optional[str]
    page_size = kwargs.pop('page_size', None)  # type: Optional[int]

    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/dataset/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/datasets/{datasetId}/definitions')
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "workspaceName": _SERIALIZER.url("workspace_name", workspace_name, 'str'),
        "datasetId": _SERIALIZER.url("dataset_id", dataset_id, 'str'),
    }

    url = _format_url_section(url, **path_format_arguments)

    # Construct parameters
    query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    if continuation_token_parameter is not None:
        query_parameters['continuationToken'] = _SERIALIZER.query("continuation_token_parameter", continuation_token_parameter, 'str')
    if page_size is not None:
        query_parameters['pageSize'] = _SERIALIZER.query("page_size", page_size, 'int')

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="GET",
        url=url,
        params=query_parameters,
        headers=header_parameters,
        **kwargs
    )


def build_update_definition_request(
    subscription_id,  # type: str
    resource_group_name,  # type: str
    workspace_name,  # type: str
    dataset_id,  # type: str
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    content_type = kwargs.pop('content_type', None)  # type: Optional[str]
    register_as_pending = kwargs.pop('register_as_pending', False)  # type: Optional[bool]
    force_update = kwargs.pop('force_update', False)  # type: Optional[bool]
    dataset_type = kwargs.pop('dataset_type', None)  # type: Optional[str]
    user_version_id = kwargs.pop('user_version_id', None)  # type: Optional[str]

    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/dataset/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/datasets/{datasetId}/definitions')
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "workspaceName": _SERIALIZER.url("workspace_name", workspace_name, 'str'),
        "datasetId": _SERIALIZER.url("dataset_id", dataset_id, 'str'),
    }

    url = _format_url_section(url, **path_format_arguments)

    # Construct parameters
    query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    if register_as_pending is not None:
        query_parameters['registerAsPending'] = _SERIALIZER.query("register_as_pending", register_as_pending, 'bool')
    if force_update is not None:
        query_parameters['forceUpdate'] = _SERIALIZER.query("force_update", force_update, 'bool')
    if dataset_type is not None:
        query_parameters['datasetType'] = _SERIALIZER.query("dataset_type", dataset_type, 'str')
    if user_version_id is not None:
        query_parameters['userVersionId'] = _SERIALIZER.query("user_version_id", user_version_id, 'str')

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    if content_type is not None:
        header_parameters['Content-Type'] = _SERIALIZER.header("content_type", content_type, 'str')
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="POST",
        url=url,
        params=query_parameters,
        headers=header_parameters,
        **kwargs
    )


def build_get_all_dataset_versions_request(
    subscription_id,  # type: str
    resource_group_name,  # type: str
    workspace_name,  # type: str
    dataset_id,  # type: str
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    continuation_token_parameter = kwargs.pop('continuation_token_parameter', None)  # type: Optional[str]
    page_size = kwargs.pop('page_size', None)  # type: Optional[int]

    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/dataset/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/datasets/{datasetId}/versions')
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "workspaceName": _SERIALIZER.url("workspace_name", workspace_name, 'str'),
        "datasetId": _SERIALIZER.url("dataset_id", dataset_id, 'str'),
    }

    url = _format_url_section(url, **path_format_arguments)

    # Construct parameters
    query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    if continuation_token_parameter is not None:
        query_parameters['continuationToken'] = _SERIALIZER.query("continuation_token_parameter", continuation_token_parameter, 'str')
    if page_size is not None:
        query_parameters['pageSize'] = _SERIALIZER.query("page_size", page_size, 'int')

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="GET",
        url=url,
        params=query_parameters,
        headers=header_parameters,
        **kwargs
    )


def build_get_dataset_by_name_request(
    subscription_id,  # type: str
    resource_group_name,  # type: str
    workspace_name,  # type: str
    dataset_name,  # type: str
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    version_id = kwargs.pop('version_id', None)  # type: Optional[str]
    include_latest_definition = kwargs.pop('include_latest_definition', True)  # type: Optional[bool]

    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/dataset/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/datasets/query/name={datasetName}')
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "workspaceName": _SERIALIZER.url("workspace_name", workspace_name, 'str'),
        "datasetName": _SERIALIZER.url("dataset_name", dataset_name, 'str'),
    }

    url = _format_url_section(url, **path_format_arguments)

    # Construct parameters
    query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    if version_id is not None:
        query_parameters['versionId'] = _SERIALIZER.query("version_id", version_id, 'str')
    if include_latest_definition is not None:
        query_parameters['includeLatestDefinition'] = _SERIALIZER.query("include_latest_definition", include_latest_definition, 'bool')

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="GET",
        url=url,
        params=query_parameters,
        headers=header_parameters,
        **kwargs
    )


def build_list_request(
    subscription_id,  # type: str
    resource_group_name,  # type: str
    workspace_name,  # type: str
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    dataset_names = kwargs.pop('dataset_names', None)  # type: Optional[List[str]]
    search_text = kwargs.pop('search_text', None)  # type: Optional[str]
    include_invisible = kwargs.pop('include_invisible', False)  # type: Optional[bool]
    status = kwargs.pop('status', None)  # type: Optional[str]
    continuation_token_parameter = kwargs.pop('continuation_token_parameter', None)  # type: Optional[str]
    page_size = kwargs.pop('page_size', None)  # type: Optional[int]
    include_latest_definition = kwargs.pop('include_latest_definition', False)  # type: Optional[bool]
    order_by = kwargs.pop('order_by', None)  # type: Optional[str]
    order_by_asc = kwargs.pop('order_by_asc', False)  # type: Optional[bool]
    dataset_types = kwargs.pop('dataset_types', None)  # type: Optional[List[str]]

    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/dataset/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/datasets')
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "workspaceName": _SERIALIZER.url("workspace_name", workspace_name, 'str'),
    }

    url = _format_url_section(url, **path_format_arguments)

    # Construct parameters
    query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    if dataset_names is not None:
        query_parameters['datasetNames'] = _SERIALIZER.query("dataset_names", dataset_names, '[str]')
    if search_text is not None:
        query_parameters['searchText'] = _SERIALIZER.query("search_text", search_text, 'str')
    if include_invisible is not None:
        query_parameters['includeInvisible'] = _SERIALIZER.query("include_invisible", include_invisible, 'bool')
    if status is not None:
        query_parameters['status'] = _SERIALIZER.query("status", status, 'str')
    if continuation_token_parameter is not None:
        query_parameters['continuationToken'] = _SERIALIZER.query("continuation_token_parameter", continuation_token_parameter, 'str')
    if page_size is not None:
        query_parameters['pageSize'] = _SERIALIZER.query("page_size", page_size, 'int')
    if include_latest_definition is not None:
        query_parameters['includeLatestDefinition'] = _SERIALIZER.query("include_latest_definition", include_latest_definition, 'bool')
    if order_by is not None:
        query_parameters['orderBy'] = _SERIALIZER.query("order_by", order_by, 'str')
    if order_by_asc is not None:
        query_parameters['orderByAsc'] = _SERIALIZER.query("order_by_asc", order_by_asc, 'bool')
    if dataset_types is not None:
        query_parameters['datasetTypes'] = _SERIALIZER.query("dataset_types", dataset_types, '[str]')

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="GET",
        url=url,
        params=query_parameters,
        headers=header_parameters,
        **kwargs
    )


def build_register_request(
    subscription_id,  # type: str
    resource_group_name,  # type: str
    workspace_name,  # type: str
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    content_type = kwargs.pop('content_type', None)  # type: Optional[str]
    register_as_pending = kwargs.pop('register_as_pending', False)  # type: Optional[bool]
    if_exists_ok = kwargs.pop('if_exists_ok', True)  # type: Optional[bool]
    update_definition_if_exists = kwargs.pop('update_definition_if_exists', False)  # type: Optional[bool]
    with_data_hash = kwargs.pop('with_data_hash', False)  # type: Optional[bool]
    user_version_id = kwargs.pop('user_version_id', None)  # type: Optional[str]

    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/dataset/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/datasets')
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "workspaceName": _SERIALIZER.url("workspace_name", workspace_name, 'str'),
    }

    url = _format_url_section(url, **path_format_arguments)

    # Construct parameters
    query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    if register_as_pending is not None:
        query_parameters['registerAsPending'] = _SERIALIZER.query("register_as_pending", register_as_pending, 'bool')
    if if_exists_ok is not None:
        query_parameters['ifExistsOk'] = _SERIALIZER.query("if_exists_ok", if_exists_ok, 'bool')
    if update_definition_if_exists is not None:
        query_parameters['updateDefinitionIfExists'] = _SERIALIZER.query("update_definition_if_exists", update_definition_if_exists, 'bool')
    if with_data_hash is not None:
        query_parameters['withDataHash'] = _SERIALIZER.query("with_data_hash", with_data_hash, 'bool')
    if user_version_id is not None:
        query_parameters['userVersionId'] = _SERIALIZER.query("user_version_id", user_version_id, 'str')

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    if content_type is not None:
        header_parameters['Content-Type'] = _SERIALIZER.header("content_type", content_type, 'str')
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="POST",
        url=url,
        params=query_parameters,
        headers=header_parameters,
        **kwargs
    )


def build_delete_all_datasets_request(
    subscription_id,  # type: str
    resource_group_name,  # type: str
    workspace_name,  # type: str
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/dataset/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/datasets')
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "workspaceName": _SERIALIZER.url("workspace_name", workspace_name, 'str'),
    }

    url = _format_url_section(url, **path_format_arguments)

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="DELETE",
        url=url,
        headers=header_parameters,
        **kwargs
    )


def build_update_dataset_request(
    subscription_id,  # type: str
    resource_group_name,  # type: str
    workspace_name,  # type: str
    dataset_id,  # type: str
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    content_type = kwargs.pop('content_type', None)  # type: Optional[str]
    force_update = kwargs.pop('force_update', False)  # type: Optional[bool]

    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/dataset/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/datasets/{datasetId}')
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "workspaceName": _SERIALIZER.url("workspace_name", workspace_name, 'str'),
        "datasetId": _SERIALIZER.url("dataset_id", dataset_id, 'str'),
    }

    url = _format_url_section(url, **path_format_arguments)

    # Construct parameters
    query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    if force_update is not None:
        query_parameters['forceUpdate'] = _SERIALIZER.query("force_update", force_update, 'bool')

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    if content_type is not None:
        header_parameters['Content-Type'] = _SERIALIZER.header("content_type", content_type, 'str')
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="PUT",
        url=url,
        params=query_parameters,
        headers=header_parameters,
        **kwargs
    )


def build_unregister_dataset_request(
    subscription_id,  # type: str
    resource_group_name,  # type: str
    workspace_name,  # type: str
    name,  # type: str
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    accept = "application/json"
    # Construct URL
    url = kwargs.pop("template_url", '/dataset/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/datasets/{name}')
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, 'str'),
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, 'str'),
        "workspaceName": _SERIALIZER.url("workspace_name", workspace_name, 'str'),
        "name": _SERIALIZER.url("name", name, 'str'),
    }

    url = _format_url_section(url, **path_format_arguments)

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="DELETE",
        url=url,
        headers=header_parameters,
        **kwargs
    )

# fmt: on
class DatasetsV1Operations(object):
    """DatasetsV1Operations operations.

    You should not instantiate this class directly. Instead, you should create a Client instance that
    instantiates it for you and attaches it as an attribute.

    :ivar models: Alias to model classes used in this operation group.
    :type models: ~azure.mgmt.machinelearningservices.models
    :param client: Client for service requests.
    :param config: Configuration of service client.
    :param serializer: An object model serializer.
    :param deserializer: An object model deserializer.
    """

    models = _models

    def __init__(self, client, config, serializer, deserializer):
        self._client = client
        self._serialize = serializer
        self._deserialize = deserializer
        self._config = config

    @distributed_trace
    def get_dataset_definition(
        self,
        subscription_id,  # type: str
        resource_group_name,  # type: str
        workspace_name,  # type: str
        dataset_id,  # type: str
        version,  # type: str
        **kwargs  # type: Any
    ):
        # type: (...) -> "_models.DatasetDefinition"
        """Get a specific dataset definition.

        :param subscription_id: The Azure Subscription ID.
        :type subscription_id: str
        :param resource_group_name: The Name of the resource group in which the workspace is located.
        :type resource_group_name: str
        :param workspace_name: The name of the workspace.
        :type workspace_name: str
        :param dataset_id:
        :type dataset_id: str
        :param version:
        :type version: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: DatasetDefinition, or the result of cls(response)
        :rtype: ~azure.mgmt.machinelearningservices.models.DatasetDefinition
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.DatasetDefinition"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_get_dataset_definition_request(
            subscription_id=subscription_id,
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            dataset_id=dataset_id,
            version=version,
            template_url=self.get_dataset_definition.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize('DatasetDefinition', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get_dataset_definition.metadata = {'url': '/dataset/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/datasets/{datasetId}/definitions/{version}'}  # type: ignore


    @distributed_trace
    def get_all_dataset_definitions(
        self,
        subscription_id,  # type: str
        resource_group_name,  # type: str
        workspace_name,  # type: str
        dataset_id,  # type: str
        continuation_token_parameter=None,  # type: Optional[str]
        page_size=None,  # type: Optional[int]
        **kwargs  # type: Any
    ):
        # type: (...) -> Iterable["_models.PaginatedDatasetDefinitionList"]
        """Get all dataset definitions for a given dataset.

        :param subscription_id: The Azure Subscription ID.
        :type subscription_id: str
        :param resource_group_name: The Name of the resource group in which the workspace is located.
        :type resource_group_name: str
        :param workspace_name: The name of the workspace.
        :type workspace_name: str
        :param dataset_id:
        :type dataset_id: str
        :param continuation_token_parameter:
        :type continuation_token_parameter: str
        :param page_size:
        :type page_size: int
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either PaginatedDatasetDefinitionList or the result of
         cls(response)
        :rtype:
         ~azure.core.paging.ItemPaged[~azure.mgmt.machinelearningservices.models.PaginatedDatasetDefinitionList]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.PaginatedDatasetDefinitionList"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        def prepare_request(next_link=None):
            if not next_link:
                
                request = build_get_all_dataset_definitions_request(
                    subscription_id=subscription_id,
                    resource_group_name=resource_group_name,
                    workspace_name=workspace_name,
                    dataset_id=dataset_id,
                    continuation_token_parameter=continuation_token_parameter,
                    page_size=page_size,
                    template_url=self.get_all_dataset_definitions.metadata['url'],
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)

            else:
                
                request = build_get_all_dataset_definitions_request(
                    subscription_id=subscription_id,
                    resource_group_name=resource_group_name,
                    workspace_name=workspace_name,
                    dataset_id=dataset_id,
                    continuation_token_parameter=continuation_token_parameter,
                    page_size=page_size,
                    template_url=next_link,
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)
                request.method = "GET"
            return request

        def extract_data(pipeline_response):
            deserialized = self._deserialize("PaginatedDatasetDefinitionList", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)
            return deserialized.next_link or None, iter(list_of_elem)

        def get_next(next_link=None):
            request = prepare_request(next_link)

            pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response


        return ItemPaged(
            get_next, extract_data
        )
    get_all_dataset_definitions.metadata = {'url': '/dataset/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/datasets/{datasetId}/definitions'}  # type: ignore

    @distributed_trace
    def update_definition(
        self,
        subscription_id,  # type: str
        resource_group_name,  # type: str
        workspace_name,  # type: str
        dataset_id,  # type: str
        register_as_pending=False,  # type: Optional[bool]
        force_update=False,  # type: Optional[bool]
        dataset_type=None,  # type: Optional[str]
        user_version_id=None,  # type: Optional[str]
        body=None,  # type: Optional["_models.DatasetDefinition"]
        **kwargs  # type: Any
    ):
        # type: (...) -> "_models.Dataset"
        """Update a dataset definition.

        :param subscription_id: The Azure Subscription ID.
        :type subscription_id: str
        :param resource_group_name: The Name of the resource group in which the workspace is located.
        :type resource_group_name: str
        :param workspace_name: The name of the workspace.
        :type workspace_name: str
        :param dataset_id:
        :type dataset_id: str
        :param register_as_pending:
        :type register_as_pending: bool
        :param force_update:
        :type force_update: bool
        :param dataset_type:
        :type dataset_type: str
        :param user_version_id:
        :type user_version_id: str
        :param body:
        :type body: ~azure.mgmt.machinelearningservices.models.DatasetDefinition
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: Dataset, or the result of cls(response)
        :rtype: ~azure.mgmt.machinelearningservices.models.Dataset
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.Dataset"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        if body is not None:
            _json = self._serialize.body(body, 'DatasetDefinition')
        else:
            _json = None

        request = build_update_definition_request(
            subscription_id=subscription_id,
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            dataset_id=dataset_id,
            content_type=content_type,
            json=_json,
            register_as_pending=register_as_pending,
            force_update=force_update,
            dataset_type=dataset_type,
            user_version_id=user_version_id,
            template_url=self.update_definition.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize('Dataset', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    update_definition.metadata = {'url': '/dataset/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/datasets/{datasetId}/definitions'}  # type: ignore


    @distributed_trace
    def get_all_dataset_versions(
        self,
        subscription_id,  # type: str
        resource_group_name,  # type: str
        workspace_name,  # type: str
        dataset_id,  # type: str
        continuation_token_parameter=None,  # type: Optional[str]
        page_size=None,  # type: Optional[int]
        **kwargs  # type: Any
    ):
        # type: (...) -> Iterable["_models.PaginatedStringList"]
        """Get all dataset versions for a given dataset.

        :param subscription_id: The Azure Subscription ID.
        :type subscription_id: str
        :param resource_group_name: The Name of the resource group in which the workspace is located.
        :type resource_group_name: str
        :param workspace_name: The name of the workspace.
        :type workspace_name: str
        :param dataset_id:
        :type dataset_id: str
        :param continuation_token_parameter:
        :type continuation_token_parameter: str
        :param page_size:
        :type page_size: int
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either PaginatedStringList or the result of cls(response)
        :rtype:
         ~azure.core.paging.ItemPaged[~azure.mgmt.machinelearningservices.models.PaginatedStringList]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.PaginatedStringList"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        def prepare_request(next_link=None):
            if not next_link:
                
                request = build_get_all_dataset_versions_request(
                    subscription_id=subscription_id,
                    resource_group_name=resource_group_name,
                    workspace_name=workspace_name,
                    dataset_id=dataset_id,
                    continuation_token_parameter=continuation_token_parameter,
                    page_size=page_size,
                    template_url=self.get_all_dataset_versions.metadata['url'],
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)

            else:
                
                request = build_get_all_dataset_versions_request(
                    subscription_id=subscription_id,
                    resource_group_name=resource_group_name,
                    workspace_name=workspace_name,
                    dataset_id=dataset_id,
                    continuation_token_parameter=continuation_token_parameter,
                    page_size=page_size,
                    template_url=next_link,
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)
                request.method = "GET"
            return request

        def extract_data(pipeline_response):
            deserialized = self._deserialize("PaginatedStringList", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)
            return deserialized.next_link or None, iter(list_of_elem)

        def get_next(next_link=None):
            request = prepare_request(next_link)

            pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response


        return ItemPaged(
            get_next, extract_data
        )
    get_all_dataset_versions.metadata = {'url': '/dataset/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/datasets/{datasetId}/versions'}  # type: ignore

    @distributed_trace
    def get_dataset_by_name(
        self,
        subscription_id,  # type: str
        resource_group_name,  # type: str
        workspace_name,  # type: str
        dataset_name,  # type: str
        version_id=None,  # type: Optional[str]
        include_latest_definition=True,  # type: Optional[bool]
        **kwargs  # type: Any
    ):
        # type: (...) -> "_models.Dataset"
        """Get a dataset for a given dataset name.

        :param subscription_id: The Azure Subscription ID.
        :type subscription_id: str
        :param resource_group_name: The Name of the resource group in which the workspace is located.
        :type resource_group_name: str
        :param workspace_name: The name of the workspace.
        :type workspace_name: str
        :param dataset_name:
        :type dataset_name: str
        :param version_id:
        :type version_id: str
        :param include_latest_definition:
        :type include_latest_definition: bool
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: Dataset, or the result of cls(response)
        :rtype: ~azure.mgmt.machinelearningservices.models.Dataset
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.Dataset"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_get_dataset_by_name_request(
            subscription_id=subscription_id,
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            dataset_name=dataset_name,
            version_id=version_id,
            include_latest_definition=include_latest_definition,
            template_url=self.get_dataset_by_name.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize('Dataset', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get_dataset_by_name.metadata = {'url': '/dataset/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/datasets/query/name={datasetName}'}  # type: ignore


    @distributed_trace
    def list(
        self,
        subscription_id,  # type: str
        resource_group_name,  # type: str
        workspace_name,  # type: str
        dataset_names=None,  # type: Optional[List[str]]
        search_text=None,  # type: Optional[str]
        include_invisible=False,  # type: Optional[bool]
        status=None,  # type: Optional[str]
        continuation_token_parameter=None,  # type: Optional[str]
        page_size=None,  # type: Optional[int]
        include_latest_definition=False,  # type: Optional[bool]
        order_by=None,  # type: Optional[str]
        order_by_asc=False,  # type: Optional[bool]
        dataset_types=None,  # type: Optional[List[str]]
        **kwargs  # type: Any
    ):
        # type: (...) -> Iterable["_models.PaginatedDatasetList"]
        """Get a list of datasets.

        :param subscription_id: The Azure Subscription ID.
        :type subscription_id: str
        :param resource_group_name: The Name of the resource group in which the workspace is located.
        :type resource_group_name: str
        :param workspace_name: The name of the workspace.
        :type workspace_name: str
        :param dataset_names:
        :type dataset_names: list[str]
        :param search_text:
        :type search_text: str
        :param include_invisible:
        :type include_invisible: bool
        :param status:
        :type status: str
        :param continuation_token_parameter:
        :type continuation_token_parameter: str
        :param page_size:
        :type page_size: int
        :param include_latest_definition:
        :type include_latest_definition: bool
        :param order_by:
        :type order_by: str
        :param order_by_asc:
        :type order_by_asc: bool
        :param dataset_types:
        :type dataset_types: list[str]
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either PaginatedDatasetList or the result of
         cls(response)
        :rtype:
         ~azure.core.paging.ItemPaged[~azure.mgmt.machinelearningservices.models.PaginatedDatasetList]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.PaginatedDatasetList"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        def prepare_request(next_link=None):
            if not next_link:
                
                request = build_list_request(
                    subscription_id=subscription_id,
                    resource_group_name=resource_group_name,
                    workspace_name=workspace_name,
                    dataset_names=dataset_names,
                    search_text=search_text,
                    include_invisible=include_invisible,
                    status=status,
                    continuation_token_parameter=continuation_token_parameter,
                    page_size=page_size,
                    include_latest_definition=include_latest_definition,
                    order_by=order_by,
                    order_by_asc=order_by_asc,
                    dataset_types=dataset_types,
                    template_url=self.list.metadata['url'],
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)

            else:
                
                request = build_list_request(
                    subscription_id=subscription_id,
                    resource_group_name=resource_group_name,
                    workspace_name=workspace_name,
                    dataset_names=dataset_names,
                    search_text=search_text,
                    include_invisible=include_invisible,
                    status=status,
                    continuation_token_parameter=continuation_token_parameter,
                    page_size=page_size,
                    include_latest_definition=include_latest_definition,
                    order_by=order_by,
                    order_by_asc=order_by_asc,
                    dataset_types=dataset_types,
                    template_url=next_link,
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)
                request.method = "GET"
            return request

        def extract_data(pipeline_response):
            deserialized = self._deserialize("PaginatedDatasetList", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)
            return deserialized.next_link or None, iter(list_of_elem)

        def get_next(next_link=None):
            request = prepare_request(next_link)

            pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response


        return ItemPaged(
            get_next, extract_data
        )
    list.metadata = {'url': '/dataset/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/datasets'}  # type: ignore

    @distributed_trace
    def register(
        self,
        subscription_id,  # type: str
        resource_group_name,  # type: str
        workspace_name,  # type: str
        register_as_pending=False,  # type: Optional[bool]
        if_exists_ok=True,  # type: Optional[bool]
        update_definition_if_exists=False,  # type: Optional[bool]
        with_data_hash=False,  # type: Optional[bool]
        user_version_id=None,  # type: Optional[str]
        body=None,  # type: Optional["_models.Dataset"]
        **kwargs  # type: Any
    ):
        # type: (...) -> "_models.Dataset"
        """Register new dataset.

        :param subscription_id: The Azure Subscription ID.
        :type subscription_id: str
        :param resource_group_name: The Name of the resource group in which the workspace is located.
        :type resource_group_name: str
        :param workspace_name: The name of the workspace.
        :type workspace_name: str
        :param register_as_pending:
        :type register_as_pending: bool
        :param if_exists_ok:
        :type if_exists_ok: bool
        :param update_definition_if_exists:
        :type update_definition_if_exists: bool
        :param with_data_hash:
        :type with_data_hash: bool
        :param user_version_id:
        :type user_version_id: str
        :param body:
        :type body: ~azure.mgmt.machinelearningservices.models.Dataset
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: Dataset, or the result of cls(response)
        :rtype: ~azure.mgmt.machinelearningservices.models.Dataset
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.Dataset"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        if body is not None:
            _json = self._serialize.body(body, 'Dataset')
        else:
            _json = None

        request = build_register_request(
            subscription_id=subscription_id,
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            content_type=content_type,
            json=_json,
            register_as_pending=register_as_pending,
            if_exists_ok=if_exists_ok,
            update_definition_if_exists=update_definition_if_exists,
            with_data_hash=with_data_hash,
            user_version_id=user_version_id,
            template_url=self.register.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize('Dataset', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    register.metadata = {'url': '/dataset/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/datasets'}  # type: ignore


    @distributed_trace
    def delete_all_datasets(
        self,
        subscription_id,  # type: str
        resource_group_name,  # type: str
        workspace_name,  # type: str
        **kwargs  # type: Any
    ):
        # type: (...) -> None
        """Unregister all datasets in the workspace.

        :param subscription_id: The Azure Subscription ID.
        :type subscription_id: str
        :param resource_group_name: The Name of the resource group in which the workspace is located.
        :type resource_group_name: str
        :param workspace_name: The name of the workspace.
        :type workspace_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_delete_all_datasets_request(
            subscription_id=subscription_id,
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            template_url=self.delete_all_datasets.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in []:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})

    delete_all_datasets.metadata = {'url': '/dataset/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/datasets'}  # type: ignore


    @distributed_trace
    def update_dataset(
        self,
        subscription_id,  # type: str
        resource_group_name,  # type: str
        workspace_name,  # type: str
        dataset_id,  # type: str
        force_update=False,  # type: Optional[bool]
        body=None,  # type: Optional["_models.Dataset"]
        **kwargs  # type: Any
    ):
        # type: (...) -> "_models.Dataset"
        """Update a dataset.

        :param subscription_id: The Azure Subscription ID.
        :type subscription_id: str
        :param resource_group_name: The Name of the resource group in which the workspace is located.
        :type resource_group_name: str
        :param workspace_name: The name of the workspace.
        :type workspace_name: str
        :param dataset_id:
        :type dataset_id: str
        :param force_update:
        :type force_update: bool
        :param body:
        :type body: ~azure.mgmt.machinelearningservices.models.Dataset
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: Dataset, or the result of cls(response)
        :rtype: ~azure.mgmt.machinelearningservices.models.Dataset
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.Dataset"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        if body is not None:
            _json = self._serialize.body(body, 'Dataset')
        else:
            _json = None

        request = build_update_dataset_request(
            subscription_id=subscription_id,
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            dataset_id=dataset_id,
            content_type=content_type,
            json=_json,
            force_update=force_update,
            template_url=self.update_dataset.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize('Dataset', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    update_dataset.metadata = {'url': '/dataset/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/datasets/{datasetId}'}  # type: ignore


    @distributed_trace
    def unregister_dataset(
        self,
        subscription_id,  # type: str
        resource_group_name,  # type: str
        workspace_name,  # type: str
        name,  # type: str
        **kwargs  # type: Any
    ):
        # type: (...) -> None
        """Unregister a dataset.

        :param subscription_id: The Azure Subscription ID.
        :type subscription_id: str
        :param resource_group_name: The Name of the resource group in which the workspace is located.
        :type resource_group_name: str
        :param workspace_name: The name of the workspace.
        :type workspace_name: str
        :param name:
        :type name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: None, or the result of cls(response)
        :rtype: None
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_unregister_dataset_request(
            subscription_id=subscription_id,
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            name=name,
            template_url=self.unregister_dataset.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in []:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})

    unregister_dataset.metadata = {'url': '/dataset/v1.0/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/datasets/{name}'}  # type: ignore




# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: ml_samples_input_output_configurations.py
DESCRIPTION:
    These samples configures for input and output of a component.
USAGE:
    python ml_samples_input_output_configurations.py

"""


class InputOutputConfigurationOptions(object):
    def ml_input_output_config(self):
        # [START configure_database]
        from azure.ai.ml.entities._inputs_outputs import Database

        # For querying a database table
        source_database = Database(query="SELECT * FROM my_table", connection="azureml:my_azuresql_connection")

        # For invoking a stored procedure with parameters
        stored_procedure_params = [
            {"name": "job", "value": "Engineer", "type": "String"},
            {"name": "department", "value": "Engineering", "type": "String"},
        ]
        source_database = Database(
            stored_procedure="SelectEmployeeByJobAndDepartment",
            stored_procedure_params=stored_procedure_params,
            connection="azureml:my_azuresql_connection",
        )

        # [END configure_database]


if __name__ == "__main__":
    sample = InputOutputConfigurationOptions()
    sample.ml_input_output_config()



import pytest

from azure.ai.ml.entities._assets import Data


@pytest.mark.unittest
@pytest.mark.core_sdk_test
class TestArtifactEntity:
    def test_eq_neq(self) -> None:
        data = Data(name="name", version="16", path=".")
        same_data = Data(name="name", version="16", path=".")
        diff_data = Data(name=data.name, version=data.version, path=data.path, description="different dataset")

        assert data == same_data != diff_data



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from typing import Any, Dict, Generator, Optional

from azure.ai.ml.entities._job.pipeline._attr_dict import _AttrDict


class PipelineJobSettings(_AttrDict):
    """Settings of PipelineJob.

    :param default_datastore: The default datastore of the pipeline.
    :type default_datastore: str
    :param default_compute: The default compute target of the pipeline.
    :type default_compute: str
    :param continue_on_step_failure: Flag indicating whether to continue pipeline execution if a step fails.
    :type continue_on_step_failure: bool
    :param force_rerun: Flag indicating whether to force rerun pipeline execution.
    :type force_rerun: bool

    .. admonition:: Example:

        .. literalinclude:: ../samples/ml_samples_pipeline_job_configurations.py
            :start-after: [START configure_pipeline_job_and_settings]
            :end-before: [END configure_pipeline_job_and_settings]
            :language: python
            :dedent: 8
            :caption: Shows how to set pipeline properties using this class.
    """

    def __init__(
        self,
        default_datastore: Optional[str] = None,
        default_compute: Optional[str] = None,
        continue_on_step_failure: Optional[bool] = None,
        force_rerun: Optional[bool] = None,
        **kwargs: Any
    ) -> None:
        self._init = True
        super().__init__()
        self.default_compute: Any = default_compute
        self.default_datastore: Any = default_datastore
        self.continue_on_step_failure = continue_on_step_failure
        self.force_rerun = force_rerun
        self.on_init = kwargs.get("on_init", None)
        self.on_finalize = kwargs.get("on_finalize", None)
        for k, v in kwargs.items():
            setattr(self, k, v)
        self._init = False

    def _get_valid_keys(self) -> Generator[str, Any, None]:
        for k, v in self.__dict__.items():
            if v is None:
                continue
            # skip private attributes inherited from _AttrDict
            if k in ["_logger", "_allowed_keys", "_init", "_key_restriction"]:
                continue
            yield k

    def _to_dict(self) -> Dict:
        result = {}
        for k in self._get_valid_keys():
            result[k] = self.__dict__[k]
        result.update(self._get_attrs())
        return result

    def _initializing(self) -> bool:
        return self._init

    def __bool__(self) -> bool:
        for _ in self._get_valid_keys():
            return True
        # _attr_dict will return False if no extra attributes are set
        return self.__len__() > 0



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=unused-argument

import logging
from typing import Any

from marshmallow import fields, post_load

from azure.ai.ml._schema._deployment.deployment import DeploymentSchema
from azure.ai.ml._schema.core.fields import ComputeField, NestedField, StringTransformedEnum
from azure.ai.ml._schema.job_resource_configuration import JobResourceConfigurationSchema
from azure.ai.ml.constants._common import BASE_PATH_CONTEXT_KEY
from azure.ai.ml.constants._deployment import BatchDeploymentType

from .model_batch_deployment_settings import ModelBatchDeploymentSettingsSchema

module_logger = logging.getLogger(__name__)


class ModelBatchDeploymentSchema(DeploymentSchema):
    compute = ComputeField(required=True)
    error_threshold = fields.Int(
        metadata={
            "description": """Error threshold, if the error count for the entire input goes above this value,\r\n
            the batch inference will be aborted. Range is [-1, int.MaxValue].\r\n
            For FileDataset, this value is the count of file failures.\r\n
            For TabularDataset, this value is the count of record failures.\r\n
            If set to -1 (the lower bound), all failures during batch inference will be ignored."""
        }
    )
    resources = NestedField(JobResourceConfigurationSchema)
    type = StringTransformedEnum(
        allowed_values=[BatchDeploymentType.PIPELINE, BatchDeploymentType.MODEL], required=False
    )

    settings = NestedField(ModelBatchDeploymentSettingsSchema)

    @post_load
    def make(self, data: Any, **kwargs: Any) -> Any:
        from azure.ai.ml.entities import ModelBatchDeployment

        return ModelBatchDeployment(base_path=self.context[BASE_PATH_CONTEXT_KEY], **data)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=protected-access

from copy import deepcopy

import yaml
from marshmallow import INCLUDE, fields, post_load, validates, ValidationError

from azure.ai.ml._schema.assets.asset import AnonymousAssetSchema
from azure.ai.ml._schema.component.component import ComponentSchema
from azure.ai.ml._schema.component.input_output import InputPortSchema
from azure.ai.ml._schema.core.schema_meta import PatchedSchemaMeta
from azure.ai.ml._schema.core.fields import FileRefField, StringTransformedEnum, NestedField
from azure.ai.ml.constants._common import BASE_PATH_CONTEXT_KEY, AssetTypes
from azure.ai.ml.constants._component import (
    ComponentSource,
    NodeType,
    DataTransferTaskType,
    DataCopyMode,
    ExternalDataType,
)


class DataTransferComponentSchemaMixin(ComponentSchema):
    type = StringTransformedEnum(allowed_values=[NodeType.DATA_TRANSFER])


class DataTransferCopyComponentSchema(DataTransferComponentSchemaMixin):
    task = StringTransformedEnum(allowed_values=[DataTransferTaskType.COPY_DATA], required=True)
    data_copy_mode = StringTransformedEnum(
        allowed_values=[DataCopyMode.MERGE_WITH_OVERWRITE, DataCopyMode.FAIL_IF_CONFLICT]
    )
    inputs = fields.Dict(
        keys=fields.Str(),
        values=NestedField(InputPortSchema),
    )

    @validates("outputs")
    def outputs_key(self, value):
        outputs_count = len(value)
        if outputs_count != 1:
            msg = "Only support single output in {}, but there're {} outputs."
            raise ValidationError(
                message=msg.format(DataTransferTaskType.COPY_DATA, outputs_count), field_name="outputs"
            )


class SinkSourceSchema(metaclass=PatchedSchemaMeta):
    type = StringTransformedEnum(
        allowed_values=[ExternalDataType.FILE_SYSTEM, ExternalDataType.DATABASE], required=True
    )


class SourceInputsSchema(metaclass=PatchedSchemaMeta):
    """
    For export task in DataTransfer, inputs type only support uri_file for database and uri_folder for filesystem.
    """

    type = StringTransformedEnum(allowed_values=[AssetTypes.URI_FOLDER, AssetTypes.URI_FILE], required=True)


class SinkOutputsSchema(metaclass=PatchedSchemaMeta):
    """
    For import task in DataTransfer, outputs type only support mltable for database and uri_folder for filesystem;
    """

    type = StringTransformedEnum(allowed_values=[AssetTypes.MLTABLE, AssetTypes.URI_FOLDER], required=True)


class DataTransferImportComponentSchema(DataTransferComponentSchemaMixin):
    task = StringTransformedEnum(allowed_values=[DataTransferTaskType.IMPORT_DATA], required=True)
    source = NestedField(SinkSourceSchema, required=True)
    outputs = fields.Dict(
        keys=fields.Str(),
        values=NestedField(SinkOutputsSchema),
    )

    @validates("inputs")
    def inputs_key(self, value):
        raise ValidationError(f"inputs field is not a valid filed in task type " f"{DataTransferTaskType.IMPORT_DATA}.")

    @validates("outputs")
    def outputs_key(self, value):
        if len(value) != 1 or value and list(value.keys())[0] != "sink":
            raise ValidationError(
                f"outputs field only support one output called sink in task type "
                f"{DataTransferTaskType.IMPORT_DATA}."
            )


class DataTransferExportComponentSchema(DataTransferComponentSchemaMixin):
    task = StringTransformedEnum(allowed_values=[DataTransferTaskType.EXPORT_DATA], required=True)
    inputs = fields.Dict(
        keys=fields.Str(),
        values=NestedField(SourceInputsSchema),
    )
    sink = NestedField(SinkSourceSchema(), required=True)

    @validates("inputs")
    def inputs_key(self, value):
        if len(value) != 1 or value and list(value.keys())[0] != "source":
            raise ValidationError(
                f"inputs field only support one input called source in task type "
                f"{DataTransferTaskType.EXPORT_DATA}."
            )

    @validates("outputs")
    def outputs_key(self, value):
        raise ValidationError(
            f"outputs field is not a valid filed in task type " f"{DataTransferTaskType.EXPORT_DATA}."
        )


class RestDataTransferCopyComponentSchema(DataTransferCopyComponentSchema):
    """When component load from rest, won't validate on name since there might
    be existing component with invalid name."""

    name = fields.Str(required=True)


class RestDataTransferImportComponentSchema(DataTransferImportComponentSchema):
    """When component load from rest, won't validate on name since there might
    be existing component with invalid name."""

    name = fields.Str(required=True)


class RestDataTransferExportComponentSchema(DataTransferExportComponentSchema):
    """When component load from rest, won't validate on name since there might
    be existing component with invalid name."""

    name = fields.Str(required=True)


class AnonymousDataTransferCopyComponentSchema(AnonymousAssetSchema, DataTransferCopyComponentSchema):
    """Anonymous data transfer copy component schema.

    Note inheritance follows order: AnonymousAssetSchema,
    AnonymousDataTransferCopyComponentSchema because we need name and version to be
    dump_only(marshmallow collects fields follows method resolution
    order).
    """

    @post_load
    def make(self, data, **kwargs):
        from azure.ai.ml.entities._component.datatransfer_component import DataTransferCopyComponent

        # Inline component will have source=YAML.JOB
        # As we only regard full separate component file as YAML.COMPONENT
        return DataTransferCopyComponent(
            base_path=self.context[BASE_PATH_CONTEXT_KEY],
            _source=kwargs.pop("_source", ComponentSource.YAML_JOB),
            **data,
        )


# pylint: disable-next=name-too-long
class AnonymousDataTransferImportComponentSchema(AnonymousAssetSchema, DataTransferImportComponentSchema):
    """Anonymous data transfer import component schema.

    Note inheritance follows order: AnonymousAssetSchema,
    DataTransferImportComponentSchema because we need name and version to be
    dump_only(marshmallow collects fields follows method resolution
    order).
    """

    @post_load
    def make(self, data, **kwargs):
        from azure.ai.ml.entities._component.datatransfer_component import DataTransferImportComponent

        # Inline component will have source=YAML.JOB
        # As we only regard full separate component file as YAML.COMPONENT
        return DataTransferImportComponent(
            base_path=self.context[BASE_PATH_CONTEXT_KEY],
            _source=kwargs.pop("_source", ComponentSource.YAML_JOB),
            **data,
        )


# pylint: disable-next=name-too-long
class AnonymousDataTransferExportComponentSchema(AnonymousAssetSchema, DataTransferExportComponentSchema):
    """Anonymous data transfer export component schema.

    Note inheritance follows order: AnonymousAssetSchema,
    DataTransferExportComponentSchema because we need name and version to be
    dump_only(marshmallow collects fields follows method resolution
    order).
    """

    @post_load
    def make(self, data, **kwargs):
        from azure.ai.ml.entities._component.datatransfer_component import DataTransferExportComponent

        # Inline component will have source=YAML.JOB
        # As we only regard full separate component file as YAML.COMPONENT
        return DataTransferExportComponent(
            base_path=self.context[BASE_PATH_CONTEXT_KEY],
            _source=kwargs.pop("_source", ComponentSource.YAML_JOB),
            **data,
        )


class DataTransferCopyComponentFileRefField(FileRefField):
    def _deserialize(self, value, attr, data, **kwargs):
        # Get component info from component yaml file.
        data = super()._deserialize(value, attr, data, **kwargs)
        component_dict = yaml.safe_load(data)
        source_path = self.context[BASE_PATH_CONTEXT_KEY] / value

        # Update base_path to parent path of component file.
        component_schema_context = deepcopy(self.context)
        component_schema_context[BASE_PATH_CONTEXT_KEY] = source_path.parent
        component = AnonymousDataTransferCopyComponentSchema(context=component_schema_context).load(
            component_dict, unknown=INCLUDE
        )
        component._source_path = source_path
        component._source = ComponentSource.YAML_COMPONENT
        return component


class DataTransferImportComponentFileRefField(FileRefField):
    def _deserialize(self, value, attr, data, **kwargs):
        # Get component info from component yaml file.
        data = super()._deserialize(value, attr, data, **kwargs)
        component_dict = yaml.safe_load(data)
        source_path = self.context[BASE_PATH_CONTEXT_KEY] / value

        # Update base_path to parent path of component file.
        component_schema_context = deepcopy(self.context)
        component_schema_context[BASE_PATH_CONTEXT_KEY] = source_path.parent
        component = AnonymousDataTransferImportComponentSchema(context=component_schema_context).load(
            component_dict, unknown=INCLUDE
        )
        component._source_path = source_path
        component._source = ComponentSource.YAML_COMPONENT
        return component


class DataTransferExportComponentFileRefField(FileRefField):
    def _deserialize(self, value, attr, data, **kwargs):
        # Get component info from component yaml file.
        data = super()._deserialize(value, attr, data, **kwargs)
        component_dict = yaml.safe_load(data)
        source_path = self.context[BASE_PATH_CONTEXT_KEY] / value

        # Update base_path to parent path of component file.
        component_schema_context = deepcopy(self.context)
        component_schema_context[BASE_PATH_CONTEXT_KEY] = source_path.parent
        component = AnonymousDataTransferExportComponentSchema(context=component_schema_context).load(
            component_dict, unknown=INCLUDE
        )
        component._source_path = source_path
        component._source = ComponentSource.YAML_COMPONENT
        return component



import os
import re
import uuid
from typing import Callable

import pytest
from devtools_testutils import AzureRecordedTestCase, is_live
from test_utilities.utils import sleep_if_live

from azure.ai.ml import MLClient, load_model
from azure.ai.ml._restclient.v2022_05_01.models import ListViewType
from azure.ai.ml.constants._common import LONG_URI_REGEX_FORMAT
from azure.ai.ml.entities._assets import Model
from azure.core.paging import ItemPaged
from pathlib import Path
from azure.ai.ml.exceptions import ValidationException


@pytest.fixture
def uuid_name() -> str:
    name = str(uuid.uuid1())
    yield name


@pytest.fixture
def artifact_path(tmpdir_factory) -> str:  # type: ignore
    file_name = tmpdir_factory.mktemp("artifact_testing").join("artifact_file.txt")
    file_name.write("content")
    return str(file_name)


# previous bodiless_matcher fixture doesn't take effect because of typo, please add it in method level if needed


def _load_flow(name: str, version: str = "42", **kwargs) -> Model:
    """Load the flow supplied with the tests."""
    return Model(
        path="./tests/test_configs/flows/basic/",
        name=name,
        type="custom_model",
        description="This is evaluator.",
        version=version,
        properties={"is-promptflow": "true", "is-evaluator": "true"},
        **kwargs,
    )


@pytest.mark.e2etest
@pytest.mark.usefixtures("recorded_test")
@pytest.mark.production_experiences_test
class TestEvaluator(AzureRecordedTestCase):
    def test_crud_file(self, client: MLClient, randstr: Callable[[], str]) -> None:
        model_name = randstr("model_name")

        model = _load_flow(model_name, version="3")
        model.name = model_name
        model = client.evaluators.create_or_update(model)
        assert model.name == model_name
        assert model.version == "3"
        assert model.description == "This is evaluator."
        assert model.type == "custom_model"
        assert "is-promptflow" in model.properties and model.properties["is-promptflow"] == "true"
        assert "is-evaluator" in model.properties and model.properties["is-evaluator"] == "true"
        assert re.match(LONG_URI_REGEX_FORMAT, model.path)

        model = client.evaluators.get(name=model.name, version="3")
        assert model.name == model_name
        assert model.version == "3"
        assert model.description == "This is evaluator."
        assert "is-promptflow" in model.properties and model.properties["is-promptflow"] == "true"
        assert "is-evaluator" in model.properties and model.properties["is-evaluator"] == "true"

        models = client.evaluators.list(name=model_name)
        assert isinstance(models, ItemPaged)
        test_model = next(iter(models), None)
        assert isinstance(test_model, Model)

        # TODO: Enable this test when listing without name will be available.
        # models = client.evaluators.list()
        # assert isinstance(models, Iterator)
        # test_model = next(iter(models), None)
        # assert isinstance(test_model, Model)

    def test_crud_evaluator_with_stage(self, client: MLClient, randstr: Callable[[], str]) -> None:
        model_name = randstr("model_prod_name")
        model = _load_flow(model_name, stage="Production", version="3")

        model = client.evaluators.create_or_update(model)
        assert model.name == model_name
        assert model.version == "3"
        assert model.description == "This is evaluator."
        assert model.type == "custom_model"
        assert model.stage == "Production"
        assert re.match(LONG_URI_REGEX_FORMAT, model.path)

        model = client.evaluators.get(name=model.name, version="3")
        assert model.name == model_name
        assert model.version == "3"
        assert model.description == "This is evaluator."
        assert model.stage == "Production"

        model_list = client.evaluators.list(name=model.name, stage="Production")
        model_stage_list = [m.stage for m in model_list if m is not None]
        assert model.stage in model_stage_list

    def test_evaluators_get_latest_label(self, client: MLClient, randstr: Callable[[], str]) -> None:
        model_name = f"model_{randstr('name')}"
        for version in ["1", "2", "3", "4"]:
            model = _load_flow(model_name, version=version)
            client.evaluators.create_or_update(model)
            assert client.evaluators.get(name=model_name, label="latest").version == version

    @pytest.mark.skip(
        "Skipping test for archive and restore as we have removed it from interface. "
        "These test will be available when the appropriate API will be enabled at "
        "GenericAssetService."
    )
    def test_evaluator_archive_restore_version(self, client: MLClient, randstr: Callable[[], str]) -> None:
        model_name = f"model_{randstr('name')}"

        versions = ["1", "2"]
        version_archived = versions[0]
        for version in versions:
            model = _load_flow(model_name, version=version)
            client.evaluators.create_or_update(model)

        def get_evaluator_list():
            # Wait for list index to update before calling list command
            sleep_if_live(30)
            model_list = client.evaluators.list(name=model_name, list_view_type=ListViewType.ACTIVE_ONLY)
            return [m.version for m in model_list if m is not None]

        assert version_archived in get_evaluator_list()
        client.evaluators.archive(name=model_name, version=version_archived)
        assert version_archived not in get_evaluator_list()
        client.evaluators.restore(name=model_name, version=version_archived)
        assert version_archived in get_evaluator_list()

    @pytest.mark.skip(reason="Task 1791832: Inefficient, possibly causing testing pipeline to time out.")
    def test_evaluator_archive_restore_container(self, client: MLClient, randstr: Callable[[], str]) -> None:
        model_name = f"model_{randstr('name')}"
        version = "1"
        model = _load_flow(model_name, version=version)

        client.evaluators.create_or_update(model)

        def get_evaluator_list():
            # Wait for list index to update before calling list command
            sleep_if_live(30)
            model_list = client.evaluators.list(list_view_type=ListViewType.ACTIVE_ONLY)
            return [m.name for m in model_list if m is not None]

        assert model_name in get_evaluator_list()
        client.evaluators.archive(name=model_name)
        assert model_name not in get_evaluator_list()
        client.evaluators.restore(name=model_name)
        assert model_name in get_evaluator_list()

    @pytest.mark.skipif(
        condition=not is_live(),
        reason="Registry uploads do not record well. Investigate later",
    )
    def test_create_get_download_evaluator_registry(
        self, registry_client: MLClient, randstr: Callable[[], str]
    ) -> None:
        model_name = randstr("model_name")
        model_version = "2"

        model_entity = _load_flow(model_name, version=model_version)
        model = registry_client.evaluators.create_or_update(model_entity)
        assert model.name == model_name
        assert model.version == model_version
        assert model.description == "This is evaluator."
        assert model.type == "custom_model"

        model_get = registry_client.evaluators.get(name=model_name, version=model_version)
        assert model == model_get
        assert model_get.name == model_name
        assert model_get.version == model_version
        assert model_get.description == "This is evaluator."
        assert model_get.type == "custom_model"

        registry_client.evaluators.download(name=model_name, version=model_version, download_path="downloaded")
        wd = os.path.join(os.getcwd(), f"downloaded/{model_name}")
        assert os.path.exists(wd)
        assert os.path.exists(f"{wd}/basic/flow.dag.yaml")

    @pytest.mark.parametrize("use_registry", [True, False])
    @pytest.mark.skipif(
        condition=not is_live(),
        reason="Registry uploads do not record well. Investigate later",
    )
    def test_list_evaluator(
        self,
        registry_client: MLClient,
        client: MLClient,
        randstr: Callable[[], str],
        use_registry: bool,
    ) -> None:
        ml_cli = registry_client if use_registry else client
        model_name = randstr("model_name")
        model_version = "1"
        model_entity = _load_flow(model_name, version=model_version)
        model = ml_cli.evaluators.create_or_update(model_entity)
        assert model.name == model_name
        assert model.version == model_version
        assert model.description == "This is evaluator."
        assert model.type == "custom_model"

        # Check that we only can create evaluators with the same name.
        model_path = Path("./tests/test_configs/model/model_full.yml")
        model_version = "2"

        model_entity = load_model(model_path)
        model_entity.name = model_name
        model_entity.version = model_version
        with pytest.raises(ValidationException) as cm:
            ml_cli.models.create_or_update(model_entity)
        assert "previous version of model was marked as promptflow evaluator" in cm.value.args[0]

        # Check that only one model was created.
        model_list = list(ml_cli.evaluators.list(model_name))
        assert len(model_list) == 1
        assert "is-promptflow" in model_list[0].properties and model_list[0].properties["is-promptflow"] == "true"
        assert "is-evaluator" in model_list[0].properties and model_list[0].properties["is-evaluator"] == "true"



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
from typing import Union

from azure.ai.ml._utils._experimental import experimental
from azure.ai.ml.entities._inputs_outputs import Input
from azure.ai.ml.constants._common import IndexInputType


# General todo: need to determine which args are required or optional when parsed out into groups like this.
# General todo: move these to more permanent locations?


# Defines stuff related to supplying inputs for an index AKA the base data.
@experimental
class IndexDataSource:
    """Base class for configs that define data that will be processed into an ML index.
    This class should not be instantiated directly. Use one of its child classes instead.

    :param input_type: A type enum describing the source of the index. Used to avoid
        direct type checking.
    :type input_type: Union[str, ~azure.ai.ml.constants._common.IndexInputType]
    """

    def __init__(self, *, input_type: Union[str, IndexInputType]):
        self.input_type = input_type


# Field bundle for creating an index from files located in a Git repo.
# TODO Does git_url need to specifically be an SSH or HTTPS style link?
# TODO What is git connection id?
@experimental
class GitSource(IndexDataSource):
    """Config class for creating an ML index from files located in a git repository.

    :param url: A link to the repository to use.
    :type url: str
    :param branch_name: The name of the branch to use from the target repository.
    :type branch_name: str
    :param connection_id: The connection ID for GitHub
    :type connection_id: str
    """

    def __init__(self, *, url: str, branch_name: str, connection_id: str):
        self.url = url
        self.branch_name = branch_name
        self.connection_id = connection_id
        super().__init__(input_type=IndexInputType.GIT)


@experimental
class LocalSource(IndexDataSource):
    """Config class for creating an ML index from a collection of local files.

    :param input_data: An input object describing the local location of index source files.
    :type input_data: ~azure.ai.ml.Input
    """

    def __init__(self, *, input_data: str):  # todo Make sure type of input_data is correct
        self.input_data = Input(type="uri_folder", path=input_data)
        super().__init__(input_type=IndexInputType.LOCAL)
